@@ -27,7 +27,12 @@
 #include "../pstl/parallel_backend.h"
 #include "function.h"
 #include "../pstl/utils.h"
-#include "scan_by_segment_impl.h"
+
+#if _ONEDPL_BACKEND_SYCL
+#    include "../pstl/hetero/algorithm_impl_hetero.h"
+#endif
+
+#include "../pstl/functional_impl.h" // for oneapi::dpl::identity
 
 namespace oneapi
 {
@@ -70,67 +75,14 @@ __pattern_inclusive_scan_by_segment(_Tag, Policy&& policy, InputIterator1 first1
     transform(policy, first1, last1 - 1, first1 + 1, _mask.get() + 1,
               oneapi::dpl::__internal::__not_pred<BinaryPredicate>(binary_pred));
 
-    inclusive_scan(::std::forward<Policy>(policy), make_zip_iterator(first2, _mask.get()),
+    inclusive_scan(std::forward<Policy>(policy), make_zip_iterator(first2, _mask.get()),
                    make_zip_iterator(first2, _mask.get()) + n, make_zip_iterator(result, _mask.get()),
-                   internal::segmented_scan_fun<ValueType, FlagType, BinaryOperator>(binary_op));
+                   oneapi::dpl::__internal::__segmented_scan_fun<ValueType, FlagType, BinaryOperator>{binary_op});
 
     return result + n;
 }
 
 #if _ONEDPL_BACKEND_SYCL
-template <typename _BackendTag, typename Policy, typename InputIterator1, typename InputIterator2,
-          typename OutputIterator, typename BinaryPredicate, typename BinaryOperator>
-OutputIterator
-__pattern_inclusive_scan_by_segment_impl(__internal::__hetero_tag<_BackendTag> __tag, Policy&& policy,
-                                         InputIterator1 first1, InputIterator1 last1, InputIterator2 first2,
-                                         OutputIterator result, BinaryPredicate binary_pred, BinaryOperator binary_op,
-                                         std::true_type /* has_known_identity */)
-{
-    using iter_value_t = typename ::std::iterator_traits<InputIterator2>::value_type;
-    iter_value_t identity = unseq_backend::__known_identity<BinaryOperator, iter_value_t>;
-    return __internal::__pattern_scan_by_segment(__tag, std::forward<Policy>(policy), first1, last1, first2, result,
-                                                 identity, binary_pred, binary_op, std::true_type{});
-}
-
-template <typename _BackendTag, typename Policy, typename InputIterator1, typename InputIterator2,
-          typename OutputIterator, typename BinaryPredicate, typename BinaryOperator>
-OutputIterator
-__pattern_inclusive_scan_by_segment_impl(__internal::__hetero_tag<_BackendTag>, Policy&& policy, InputIterator1 first1,
-                                         InputIterator1 last1, InputIterator2 first2, OutputIterator result,
-                                         BinaryPredicate binary_pred, BinaryOperator binary_op,
-                                         std::false_type /* has_known_identity */)
-{
-
-    typedef unsigned int FlagType;
-    typedef typename ::std::iterator_traits<InputIterator2>::value_type ValueType;
-
-    const auto n = ::std::distance(first1, last1);
-
-    // Check for empty element ranges
-    if (n <= 0)
-        return result;
-
-    FlagType initial_mask = 1;
-
-    oneapi::dpl::__par_backend_hetero::__buffer<FlagType> _mask(n);
-    {
-        auto mask_buf = _mask.get_buffer();
-        auto mask = mask_buf.get_host_access(sycl::read_write);
-
-        mask[0] = initial_mask;
-    }
-
-    transform(::std::forward<Policy>(policy), first1, last1 - 1, first1 + 1, _mask.get() + 1,
-              oneapi::dpl::__internal::__not_pred<BinaryPredicate>(binary_pred));
-
-    auto policy1 = oneapi::dpl::__par_backend_hetero::make_wrapped_policy<InclusiveScan1>(policy);
-    transform_inclusive_scan(::std::move(policy1), make_zip_iterator(first2, _mask.get()),
-                             make_zip_iterator(first2, _mask.get()) + n, make_zip_iterator(result, _mask.get()),
-                             internal::segmented_scan_fun<ValueType, FlagType, BinaryOperator>(binary_op),
-                             oneapi::dpl::__internal::__no_op());
-    return result + n;
-}
-
 template <typename _BackendTag, typename Policy, typename InputIterator1, typename InputIterator2,
           typename OutputIterator, typename BinaryPredicate, typename BinaryOperator>
 OutputIterator
@@ -138,10 +90,8 @@ __pattern_inclusive_scan_by_segment(__internal::__hetero_tag<_BackendTag> __tag,
                                     InputIterator1 last1, InputIterator2 first2, OutputIterator result,
                                     BinaryPredicate binary_pred, BinaryOperator binary_op)
 {
-    return __internal::__pattern_inclusive_scan_by_segment_impl(
-        __tag, ::std::forward<Policy>(policy), first1, last1, first2, result, binary_pred, binary_op,
-        typename unseq_backend::__has_known_identity<
-            BinaryOperator, typename ::std::iterator_traits<InputIterator2>::value_type>::type{});
+    return __pattern_scan_by_segment(__tag, std::forward<Policy>(policy), first1, last1, first2, result, binary_pred,
+                                     binary_op, /*_Inclusive*/ std::true_type{});
 }
 
 #endif
