@@ -79,9 +79,15 @@ typename __difference<_Ip>::__type
 __calculate_input_sequence_length(const _Ip __first, const _Ip __last, const _Sp __stride)
 {
     assert(__stride != 0);
-
-    return (__stride > 0) ? ((__last - __first + (__stride - 1)) / __stride)
-                          : ((__first - __last - (__stride + 1)) / -__stride);
+    if constexpr (std::is_unsigned_v<_Sp>)
+    {
+        return (__last - __first + (__stride - 1)) / __stride;
+    }
+    else
+    {
+        return (__stride > 0) ? ((__last - __first + (__stride - 1)) / __stride)
+                              : ((__first - __last - (__stride + 1)) / -__stride);
+    }
 }
 
 template <typename _Ip>
@@ -244,7 +250,7 @@ __execute_loop_strided(_Ip __first, _Ip __last, _Function __f, _Sp __stride, _Pa
 
     if (__stride > 0)
     {
-        for (_IndexType __i = 0; __first != __last; ++__first, ++__i)
+        for (_IndexType __i = 0; __first != __last; ++__first, (void)++__i)
         {
             if (__i % __stride == 0)
             {
@@ -255,7 +261,7 @@ __execute_loop_strided(_Ip __first, _Ip __last, _Function __f, _Sp __stride, _Pa
     }
     else
     {
-        for (_IndexType __i = 0; __first != __last; --__first, ++__i)
+        for (_IndexType __i = 0; __first != __last; --__first, (void)++__i)
         {
             if (__i % __stride == 0)
             {
@@ -279,7 +285,7 @@ __execute_loop_strided(_Ip __first, _Ip __last, _Function __f, _Sp __stride, _Pa
 
     assert(__stride > 0);
 
-    for (_IndexType __i = 0; __first != __last; ++__first, ++__i)
+    for (_IndexType __i = 0; __first != __last; ++__first, (void)++__i)
     {
         if (__i % __stride == 0)
         {
@@ -304,7 +310,7 @@ __pattern_for_loop(_ExecutionPolicy&&, _Ip __first, _Ip __last, _Function __f, _
     __index_type __ordinal_position = 0;
 
     // Avoid check for i % stride on each iteration for the most common case.
-    for (; __first != __last; ++__first, ++__ordinal_position)
+    for (; __first != __last; ++__first, (void)++__ordinal_position)
         __pack.__apply_func(__f, __first, __ordinal_position);
 
     __pack.__finalize(__ordinal_position);
@@ -324,7 +330,7 @@ __pattern_for_loop(_ExecutionPolicy&&, _Ip __first, _Ip __last, _Function __f, _
     if (__stride == 1)
     {
         // Avoid check for i % stride on each iteration for the most common case.
-        for (; __first != __last; ++__first, ++__ordinal_position)
+        for (; __first != __last; ++__first, (void)++__ordinal_position)
             __pack.__apply_func(__f, __first, __ordinal_position);
     }
     else
@@ -496,14 +502,14 @@ struct __use_par_vec_helper
 };
 
 // Special versions for for_loop: handles both iterators and integral types(treated as random access iterators)
-template <typename _ExecutionPolicy, typename _Ip>
+template <typename _Ip, typename _ExecutionPolicy>
 auto
 __use_vectorization(_ExecutionPolicy&& __exec)
 {
     return __use_par_vec_helper<_Ip>::__use_vector(::std::forward<_ExecutionPolicy>(__exec));
 }
 
-template <typename _ExecutionPolicy, typename _Ip>
+template <typename _Ip, typename _ExecutionPolicy>
 auto
 __use_parallelization(_ExecutionPolicy&& __exec)
 {
@@ -516,11 +522,10 @@ void
 __for_loop_impl(_ExecutionPolicy&& __exec, _Ip __start, _Ip __finish, _Fp&& __f, _Sp __stride,
                 ::std::tuple<_Rest...>&& __t, ::std::index_sequence<_Is...>)
 {
-    oneapi::dpl::__internal::__pattern_for_loop(
-        ::std::forward<_ExecutionPolicy>(__exec), __start, __finish, __f, __stride,
-        oneapi::dpl::__internal::__use_vectorization<_ExecutionPolicy, _Ip>(__exec),
-        oneapi::dpl::__internal::__use_parallelization<_ExecutionPolicy, _Ip>(__exec),
-        ::std::get<_Is>(::std::move(__t))...);
+    oneapi::dpl::__internal::__pattern_for_loop(::std::forward<_ExecutionPolicy>(__exec), __start, __finish, __f,
+                                                __stride, oneapi::dpl::__internal::__use_vectorization<_Ip>(__exec),
+                                                oneapi::dpl::__internal::__use_parallelization<_Ip>(__exec),
+                                                ::std::get<_Is>(::std::move(__t))...);
 }
 
 template <typename _ExecutionPolicy, typename _Ip, typename _Size, typename _Fp, typename _Sp, typename... _Rest,
@@ -529,11 +534,10 @@ void
 __for_loop_n_impl(_ExecutionPolicy&& __exec, _Ip __start, _Size __n, _Fp&& __f, _Sp __stride,
                   ::std::tuple<_Rest...>&& __t, ::std::index_sequence<_Is...>)
 {
-    oneapi::dpl::__internal::__pattern_for_loop_n(
-        ::std::forward<_ExecutionPolicy>(__exec), __start, __n, __f, __stride,
-        oneapi::dpl::__internal::__use_vectorization<_ExecutionPolicy, _Ip>(__exec),
-        oneapi::dpl::__internal::__use_parallelization<_ExecutionPolicy, _Ip>(__exec),
-        ::std::get<_Is>(::std::move(__t))...);
+    oneapi::dpl::__internal::__pattern_for_loop_n(::std::forward<_ExecutionPolicy>(__exec), __start, __n, __f, __stride,
+                                                  oneapi::dpl::__internal::__use_vectorization<_Ip>(__exec),
+                                                  oneapi::dpl::__internal::__use_parallelization<_Ip>(__exec),
+                                                  ::std::get<_Is>(::std::move(__t))...);
 }
 
 template <typename _ExecutionPolicy, typename _Ip, typename _Sp, typename... _Rest>
