@@ -23,7 +23,8 @@ main()
 {
 #if TEST_DPCPP_BACKEND_PRESENT
 
-    sycl::queue q = TestUtils::get_test_queue();
+    auto policy = TestUtils::get_dpcpp_test_policy();
+    sycl::queue q = policy.queue();
 
     constexpr std::size_t n = 100;
 
@@ -33,7 +34,7 @@ main()
     allocator alloc(q);
     std::vector<T, allocator> data(n, 1, alloc);
 
-    auto f = oneapi::dpl::experimental::reduce_async(TestUtils::make_device_policy(q), data.begin(), data.end());
+    auto f = oneapi::dpl::experimental::reduce_async(policy, data.begin(), data.end());
     f.wait();
 
 #endif // TEST_DPCPP_BACKEND_PRESENT
