@@ -13,6 +13,8 @@
 //
 //===----------------------------------------------------------------------===//
 
+#include "support/test_config.h"
+
 #include <oneapi/dpl/execution>
 #include <oneapi/dpl/algorithm>
 #include <oneapi/dpl/functional>
@@ -21,30 +23,8 @@
 #include "support/utils.h"
 
 #include <algorithm>
-#include <functional>
 #include <iterator>
 #include <vector>
-#include "support/test_config.h"
-
-#if TEST_DPCPP_BACKEND_PRESENT
-#include <CL/sycl.hpp>
-#endif
-
-template <typename Policy, typename NewName>
-struct rebind_policy { using type = Policy; };
-
-#if TEST_DPCPP_BACKEND_PRESENT
-template <typename KernelName, typename NewName>
-struct rebind_policy<oneapi::dpl::execution::device_policy<KernelName>, NewName>
-{ using type = oneapi::dpl::execution::device_policy<NewName>; };
-
-#if ONEDPL_FPGA_DEVICE
-template <unsigned int factor, typename KernelName, typename NewName>
-struct rebind_policy<oneapi::dpl::execution::fpga_policy<factor, KernelName>, NewName>
-{  using type = oneapi::dpl::execution::fpga_policy<factor, NewName>; };
-#endif
-#endif
-
 
 using oneapi::dpl::counting_iterator;
 using oneapi::dpl::discard_iterator;
