@@ -35,14 +35,14 @@ DEFINE_TEST(test_reduce)
         host_keys.update_data();
 
         // without initial value
-        auto result1 = ::std::reduce(make_new_policy<new_kernel_name<Policy, 0>>(exec), first1 + (n / 3), first1 + (n / 2));
+        auto result1 = std::reduce(CLONE_TEST_POLICY_IDX(exec, 0), first1 + (n / 3), first1 + (n / 2));
         wait_and_throw(exec);
 
         EXPECT_TRUE(result1 == value * (n / 2 - n / 3), "wrong effect from reduce (1)");
 
         // with initial value
         auto init = T1(42);
-        auto result2 = ::std::reduce(make_new_policy<new_kernel_name<Policy, 0>>(exec), first1 + (n / 3), first1 + (n / 2), init);
+        auto result2 = std::reduce(CLONE_TEST_POLICY_IDX(exec, 0), first1 + (n / 3), first1 + (n / 2), init);
         wait_and_throw(exec);
 
         EXPECT_TRUE(result2 == init + value * (n / 2 - n / 3), "wrong effect from reduce (2)");
@@ -65,7 +65,7 @@ DEFINE_TEST(test_transform_reduce_unary)
         ::std::fill(host_keys.get(), host_keys.get() + n, value);
         host_keys.update_data();
 
-        auto result = ::std::transform_reduce(make_new_policy<new_kernel_name<Policy, 0>>(exec), first1, last1, T1(42),
+        auto result = std::transform_reduce(CLONE_TEST_POLICY_IDX(exec, 0), first1, last1, T1(42),
                                             Plus(), ::std::negate<T1>());
         wait_and_throw(exec);
 
@@ -98,21 +98,14 @@ DEFINE_TEST(test_min_element)
         }
         host_keys.update_data();
 
-        auto result_min = ::std::min_element(make_new_policy<new_kernel_name<Policy, 0>>(exec), first, last);
+        auto result_min = std::min_element(CLONE_TEST_POLICY_IDX(exec, 0), first, last);
         wait_and_throw(exec);
 
         host_keys.retrieve_data();
 
         auto expected_min = ::std::min_element(host_keys.get(), host_keys.get() + n);
 
-        EXPECT_TRUE(result_min - first == expected_min - host_keys.get(),
-                    "wrong effect from min_element");
-#if _ONEDPL_DEBUG_SYCL
-        ::std::cout << "got: " << *(host_keys.get() + (result_min - first)) << "["
-                    << result_min - first << "], "
-                    << "expected: " << *(host_keys.get() + (expected_min - host_keys.get())) << "["
-                    << expected_min - host_keys.get() << "]" << ::std::endl;
-#endif // _ONEDPL_DEBUG_SYCL
+        EXPECT_EQ(expected_min - host_keys.get(), result_min - first, "wrong effect from min_element");
     }
 };
 
@@ -142,17 +135,12 @@ DEFINE_TEST(test_max_element)
 
         auto expected_max_offset = ::std::max_element(host_keys.get(), host_keys.get() + n) - host_keys.get();
 
-        auto result_max_offset = ::std::max_element(make_new_policy<new_kernel_name<Policy, 0>>(exec), first, last) - first;
+        auto result_max_offset = std::max_element(CLONE_TEST_POLICY_IDX(exec, 0), first, last) - first;
         wait_and_throw(exec);
 
         host_keys.retrieve_data();
 
-        EXPECT_TRUE(result_max_offset == expected_max_offset, "wrong effect from max_element");
-#if _ONEDPL_DEBUG_SYCL
-        ::std::cout << "got: "      << *(host_keys.get() + result_max_offset)   << "[" << result_max_offset   << "], "
-                    << "expected: " << *(host_keys.get() + expected_max_offset) << "[" << expected_max_offset << "]"
-                    << ::std::endl;
-#endif // _ONEDPL_DEBUG_SYCL
+        EXPECT_EQ(expected_max_offset, result_max_offset, "wrong effect from max_element");
     }
 };
 
@@ -186,24 +174,14 @@ DEFINE_TEST(test_minmax_element)
         auto expected_max = expected.second - host_keys.get();
         ::std::pair<Size, Size> expected_offset = { expected_min, expected_max };
 
-        auto result = ::std::minmax_element(make_new_policy<new_kernel_name<Policy, 0>>(exec), first, last);
+        auto result = std::minmax_element(CLONE_TEST_POLICY_IDX(exec, 0), first, last);
         auto result_min = result.first - first;
         auto result_max = result.second - first;
 
         wait_and_throw(exec);
 
-        EXPECT_TRUE(result_min == expected_min && result_max == expected_max, "wrong effect from minmax_element");
-        if (!(result_min == expected_min && result_max == expected_max))
-        {
-            host_keys.retrieve_data();
-
-            auto got_min = host_keys.get() + (result.first - first);
-            auto got_max = host_keys.get() + (result.second - first);
-            ::std::cout << "MIN got: " << got_min << "[" << result_min << "], "
-                        << "expected: " << *(host_keys.get() + expected_offset.first) << "[" << expected_min << "]" << ::std::endl;
-            ::std::cout << "MAX got: " << got_max << "[" << result_max << "], "
-                        << "expected: " << *(host_keys.get() + expected_offset.second) << "[" << expected_max << "]" << ::std::endl;
-        }
+        EXPECT_EQ(expected_min, result_min,  "wrong effect from minmax_element: result_min");
+        EXPECT_EQ(expected_max, result_max,  "wrong effect from minmax_element: result_max");
     }
 };
 
@@ -226,36 +204,27 @@ DEFINE_TEST(test_count)
 
         // check when arbitrary should be counted
         ReturnType expected = (n - 1) / 10 + 1;
-        ReturnType result = ::std::count(make_new_policy<new_kernel_name<Policy, 0>>(exec), first, last, ValueType{0});
+        ReturnType result = std::count(CLONE_TEST_POLICY_IDX(exec, 0), first, last, ValueType{0});
         wait_and_throw(exec);
 
-        EXPECT_TRUE(result == expected, "wrong effect from count (Test #1 arbitrary to count)");
-#if _ONEDPL_DEBUG_SYCL
-        ::std::cout << "got " << result << ", expected " << expected << ::std::endl;
-#endif // _ONEDPL_DEBUG_SYCL
+        EXPECT_EQ(expected, result, "wrong effect from count (Test #1 arbitrary to count)");
 
         // check when none should be counted
         expected = 0;
-        result = ::std::count(make_new_policy<new_kernel_name<Policy, 0>>(exec), first, last, ValueType{12});
+        result = std::count(CLONE_TEST_POLICY_IDX(exec, 0), first, last, ValueType{12});
         wait_and_throw(exec);
 
-        EXPECT_TRUE(result == expected, "wrong effect from count (Test #2 none to count)");
-#if _ONEDPL_DEBUG_SYCL
-        ::std::cout << "got " << result << ", expected " << expected << ::std::endl;
-#endif // _ONEDPL_DEBUG_SYCL
+        EXPECT_EQ(expected, result, "wrong effect from count (Test #2 none to count)");
 
         // check when all should be counted
         ::std::fill(host_keys.get(), host_keys.get() + n, ValueType{7});
         host_keys.update_data();
 
         expected = n;
-        result = ::std::count(make_new_policy<new_kernel_name<Policy, 0>>(exec), first, last, ValueType{7});
+        result = std::count(CLONE_TEST_POLICY_IDX(exec, 0), first, last, ValueType{7});
         wait_and_throw(exec);
 
-        EXPECT_TRUE(result == expected, "wrong effect from count (Test #3 all to count)");
-#if _ONEDPL_DEBUG_SYCL
-        ::std::cout << "got " << result << ", expected " << expected << ::std::endl;
-#endif // _ONEDPL_DEBUG_SYCL
+        EXPECT_EQ(expected, result, "wrong effect from count (Test #3 all to count)");
     }
 };
 
@@ -278,36 +247,24 @@ DEFINE_TEST(test_count_if)
 
         // check when arbitrary should be counted
         ReturnType expected = (n - 1) / 10 + 1;
-        ReturnType result = ::std::count_if(make_new_policy<new_kernel_name<Policy, 0>>(exec), first, last,
-                                            [](ValueType const& value) { return value % 10 == 0; });
+        ReturnType result = std::count_if(CLONE_TEST_POLICY_IDX(exec, 0), first, last, TestUtils::IsMultipleOf<ValueType>{10});
         wait_and_throw(exec);
 
-        EXPECT_TRUE(result == expected, "wrong effect from count_if (Test #1 arbitrary to count)");
-#if _ONEDPL_DEBUG_SYCL
-        ::std::cout << "got " << result << ", expected " << expected << ::std::endl;
-#endif // _ONEDPL_DEBUG_SYCL
+        EXPECT_EQ(expected, result, "wrong effect from count_if (Test #1 arbitrary to count)");
 
         // check when none should be counted
         expected = 0;
-        result = ::std::count_if(make_new_policy<new_kernel_name<Policy, 1>>(exec), first, last,
-                                 [](ValueType const& value) { return value > 10; });
+        result = std::count_if(CLONE_TEST_POLICY_IDX(exec, 1), first, last, TestUtils::IsGreatThan<ValueType>{10});
         wait_and_throw(exec);
 
-        EXPECT_TRUE(result == expected, "wrong effect from count_if (Test #2 none to count)");
-#if _ONEDPL_DEBUG_SYCL
-        ::std::cout << "got " << result << ", expected " << expected << ::std::endl;
-#endif // _ONEDPL_DEBUG_SYCL
+        EXPECT_EQ(expected, result, "wrong effect from count_if (Test #2 none to count)");
 
         // check when all should be counted
         expected = n;
-        result = ::std::count_if(make_new_policy<new_kernel_name<Policy, 2>>(exec), first, last,
-                                 [](ValueType const& value) { return value < 10; });
+        result = std::count_if(CLONE_TEST_POLICY_IDX(exec, 2), first, last, TestUtils::IsLessThan<ValueType>{10});
         wait_and_throw(exec);
 
-        EXPECT_TRUE(result == expected, "wrong effect from count_if (Test #3 all to count)");
-#if _ONEDPL_DEBUG_SYCL
-        ::std::cout << "got " << result << ", expected " << expected << ::std::endl;
-#endif // _ONEDPL_DEBUG_SYCL
+        EXPECT_EQ(expected, result, "wrong effect from count_if (Test #3 all to count)");
     }
 };
 
@@ -326,8 +283,8 @@ DEFINE_TEST(test_is_partitioned)
         if (n < 2)
             return;
 
-        auto less_than = [](const ValueType& value) -> bool { return value < 10; };
-        auto is_odd = [](const ValueType& value) -> bool { return value % 2; };
+        auto less_than = TestUtils::IsLessThan<ValueType>{10};
+        auto is_odd = TestUtils::IsOdd<ValueType>{};
 
         bool expected_bool_less_then = false;
         bool expected_bool_is_odd = false;
@@ -339,25 +296,25 @@ DEFINE_TEST(test_is_partitioned)
         host_keys.update_data();
 
         // check sorted
-        bool result_bool = ::std::is_partitioned(make_new_policy<new_kernel_name<Policy, 0>>(exec), first, last, less_than);
+        bool result_bool = std::is_partitioned(CLONE_TEST_POLICY_IDX(exec, 0), first, last, less_than);
         wait_and_throw(exec);
 
-        EXPECT_TRUE(result_bool == expected_bool_less_then, "wrong effect from is_partitioned (Test #1 less than)");
+        EXPECT_EQ(expected_bool_less_then, result_bool, "wrong effect from is_partitioned (Test #1 less than)");
 
-        result_bool = ::std::is_partitioned(make_new_policy<new_kernel_name<Policy, 1>>(exec), first, last, is_odd);
+        result_bool = std::is_partitioned(CLONE_TEST_POLICY_IDX(exec, 1), first, last, is_odd);
         wait_and_throw(exec);
 
-        EXPECT_TRUE(result_bool == expected_bool_is_odd, "wrong effect from is_partitioned (Test #2 is odd)");
+        EXPECT_EQ(expected_bool_is_odd, result_bool, "wrong effect from is_partitioned (Test #2 is odd)");
 
         // The code as below was added to prevent accessor destruction working with host memory
         ::std::partition(host_keys.get(), host_keys.get() + n, is_odd);
         expected_bool_is_odd = ::std::is_partitioned(host_keys.get(), host_keys.get() + n, is_odd);
         host_keys.update_data();
 
-        result_bool = ::std::is_partitioned(make_new_policy<new_kernel_name<Policy, 2>>(exec), first, last, is_odd);
+        result_bool = std::is_partitioned(CLONE_TEST_POLICY_IDX(exec, 2), first, last, is_odd);
         wait_and_throw(exec);
 
-        EXPECT_TRUE(result_bool == expected_bool_is_odd,
+        EXPECT_EQ(expected_bool_is_odd, result_bool,
                     "wrong effect from is_partitioned (Test #3 is odd after partition)");
     }
 };
@@ -379,7 +336,7 @@ DEFINE_TEST(test_transform_reduce_binary)
         host_keys.update_data();
 
         auto result =
-            ::std::transform_reduce(make_new_policy<new_kernel_name<Policy, 0>>(exec), first1, last1, first1, T1(42));
+            std::transform_reduce(CLONE_TEST_POLICY_IDX(exec, 0), first1, last1, first1, T1(42));
         wait_and_throw(exec);
 
         EXPECT_TRUE(result == n + 42, "wrong effect from transform_reduce (2 binary)");
@@ -411,34 +368,28 @@ DEFINE_TEST(test_lexicographical_compare)
             update_data(host_keys, host_vals);
         }
 
-        auto comp = [](ValueType const& first, ValueType const& second) { return first < second; };
+        auto comp = TestUtils::IsLess<const ValueType&>{};
 
         // CHECK 1.1: S1 == S2 && len(S1) == len(S2)
-        bool is_less_res = ::std::lexicographical_compare(make_new_policy<new_kernel_name<Policy, 0>>(exec), first1,
+        bool is_less_res = std::lexicographical_compare(CLONE_TEST_POLICY_IDX(exec, 0), first1,
                                                           last1, first2, last2, comp);
         wait_and_throw(exec);
 
-        if (is_less_res != 0)
-            ::std::cout << "N=" << n << ": got " << is_less_res << ", expected 0" << ::std::endl;
-        EXPECT_TRUE(is_less_res == 0, "wrong effect from lex_compare Test 1.1: S1 == S2 && len(S1) == len(S2)");
+        EXPECT_EQ(0, is_less_res, "wrong effect from lex_compare Test 1.1: S1 == S2 && len(S1) == len(S2)");
 
         // CHECK 1.2: S1 == S2 && len(S1) < len(S2)
-        is_less_res = ::std::lexicographical_compare(make_new_policy<new_kernel_name<Policy, 1>>(exec), first1, last1 - 1,
+        is_less_res = std::lexicographical_compare(CLONE_TEST_POLICY_IDX(exec, 1), first1, last1 - 1,
                                                    first2, last2, comp);
         wait_and_throw(exec);
 
-        if (is_less_res != 1)
-            ::std::cout << "N=" << n << ": got " << is_less_res << ", expected 1" << ::std::endl;
-        EXPECT_TRUE(is_less_res == 1, "wrong effect from lex_compare Test 1.2: S1 == S2 && len(S1) < len(S2)");
+        EXPECT_EQ(1, is_less_res, "wrong effect from lex_compare Test 1.2: S1 == S2 && len(S1) < len(S2)");
 
         // CHECK 1.3: S1 == S2 && len(S1) > len(S2)
-        is_less_res = ::std::lexicographical_compare(make_new_policy<new_kernel_name<Policy, 2>>(exec), first1, last1, first2,
+        is_less_res = std::lexicographical_compare(CLONE_TEST_POLICY_IDX(exec, 2), first1, last1, first2,
                                                    last2 - 1, comp);
         wait_and_throw(exec);
 
-        if (is_less_res != 0)
-            ::std::cout << "N=" << n << ": got " << is_less_res << ", expected 0" << ::std::endl;
-        EXPECT_TRUE(is_less_res == 0, "wrong effect from lex_compare Test 1.3: S1 == S2 && len(S1) > len(S2)");
+        EXPECT_EQ(0, is_less_res, "wrong effect from lex_compare Test 1.3: S1 == S2 && len(S1) > len(S2)");
 
         if (n > 1)
         {
@@ -447,25 +398,19 @@ DEFINE_TEST(test_lexicographical_compare)
         }
 
         // CHECK 2.1: S1 < S2 (PRE-LAST ELEMENT) && len(S1) == len(S2)
-        is_less_res = ::std::lexicographical_compare(make_new_policy<new_kernel_name<Policy, 3>>(exec), first1, last1, first2,
+        is_less_res = std::lexicographical_compare(CLONE_TEST_POLICY_IDX(exec, 3), first1, last1, first2,
                                                    last2, comp);
         wait_and_throw(exec);
 
         bool is_less_exp = n > 1 ? 1 : 0;
-        if (is_less_res != is_less_exp)
-            ::std::cout << "N=" << n << ": got " << is_less_res << ", expected " << is_less_exp << ::std::endl;
-        EXPECT_TRUE(is_less_res == is_less_exp,
-                    "wrong effect from lex_compare Test 2.1: S1 < S2 (PRE-LAST) && len(S1) == len(S2)");
+        EXPECT_EQ(is_less_exp, is_less_res, "wrong effect from lex_compare Test 2.1: S1 < S2 (PRE-LAST) && len(S1) == len(S2)");
 
         // CHECK 2.2: S1 < S2 (PRE-LAST ELEMENT) && len(S1) > len(S2)
-        is_less_res = ::std::lexicographical_compare(make_new_policy<new_kernel_name<Policy, 4>>(exec), first1, last1, first2,
+        is_less_res = std::lexicographical_compare(CLONE_TEST_POLICY_IDX(exec, 4), first1, last1, first2,
                                                    last2 - 1, comp);
         wait_and_throw(exec);
 
-        if (is_less_res != is_less_exp)
-            ::std::cout << "N=" << n << ": got " << is_less_res << ", expected " << is_less_exp << ::std::endl;
-        EXPECT_TRUE(is_less_res == is_less_exp,
-                    "wrong effect from lex_compare Test 2.2: S1 < S2 (PRE-LAST) && len(S1) > len(S2)");
+        EXPECT_EQ(is_less_exp, is_less_res, "wrong effect from lex_compare Test 2.2: S1 < S2 (PRE-LAST) && len(S1) > len(S2)");
 
         if (n > 1)
         {
@@ -474,73 +419,57 @@ DEFINE_TEST(test_lexicographical_compare)
         }
 
         // CHECK 3.1: S1 > S2 (PRE-LAST ELEMENT) && len(S1) == len(S2)
-        is_less_res = ::std::lexicographical_compare(make_new_policy<new_kernel_name<Policy, 5>>(exec), first1, last1, first2,
+        is_less_res = std::lexicographical_compare(CLONE_TEST_POLICY_IDX(exec, 5), first1, last1, first2,
                                                    last2, comp);
         wait_and_throw(exec);
 
-        if (is_less_res != 0)
-            ::std::cout << "N=" << n << ": got " << is_less_res << ", expected 0" << ::std::endl;
-        EXPECT_TRUE(is_less_res == 0,
-                    "wrong effect from lex_compare Test 3.1: S1 > S2 (PRE-LAST) && len(S1) == len(S2)");
+        EXPECT_EQ(0, is_less_res, "wrong effect from lex_compare Test 3.1: S1 > S2 (PRE-LAST) && len(S1) == len(S2)");
 
         // CHECK 3.2: S1 > S2 (PRE-LAST ELEMENT) && len(S1) < len(S2)
-        is_less_res = ::std::lexicographical_compare(make_new_policy<new_kernel_name<Policy, 6>>(exec), first1, last1 - 1,
+        is_less_res = std::lexicographical_compare(CLONE_TEST_POLICY_IDX(exec, 6), first1, last1 - 1,
                                                    first2, last2, comp);
         wait_and_throw(exec);
 
         is_less_exp = n > 1 ? 0 : 1;
-        if (is_less_res != is_less_exp)
-            ::std::cout << "N=" << n << ": got " << is_less_res << ", expected " << is_less_exp << ::std::endl;
-        EXPECT_TRUE(is_less_res == is_less_exp,
-                    "wrong effect from lex_compare Test 3.2: S1 > S2 (PRE-LAST) && len(S1) < len(S2)");
+        EXPECT_EQ(is_less_exp, is_less_res, "wrong effect from lex_compare Test 3.2: S1 > S2 (PRE-LAST) && len(S1) < len(S2)");
         {
             *host_vals.get() = 444;
             host_vals.update_data();
         }
 
         // CHECK 4.1: S1 < S2 (FIRST ELEMENT) && len(S1) == len(S2)
-        is_less_res = ::std::lexicographical_compare(make_new_policy<new_kernel_name<Policy, 7>>(exec), first1, last1, first2,
+        is_less_res = std::lexicographical_compare(CLONE_TEST_POLICY_IDX(exec, 7), first1, last1, first2,
                                                    last2, comp);
         wait_and_throw(exec);
 
-        if (is_less_res != 1)
-            ::std::cout << "N=" << n << ": got " << is_less_res << ", expected 1" << ::std::endl;
-        EXPECT_TRUE(is_less_res == 1, "wrong effect from lex_compare Test 4.1: S1 < S2 (FIRST) && len(S1) == len(S2)");
+        EXPECT_EQ(1, is_less_res, "wrong effect from lex_compare Test 4.1: S1 < S2 (FIRST) && len(S1) == len(S2)");
 
         // CHECK 4.2: S1 < S2 (FIRST ELEMENT) && len(S1) > len(S2)
-        is_less_res = ::std::lexicographical_compare(make_new_policy<new_kernel_name<Policy, 8>>(exec), first1, last1, first2,
+        is_less_res = std::lexicographical_compare(CLONE_TEST_POLICY_IDX(exec, 8), first1, last1, first2,
                                                    last2 - 1, comp);
         wait_and_throw(exec);
 
         is_less_exp = n > 1 ? 1 : 0;
-        if (is_less_res != is_less_exp)
-            ::std::cout << "N=" << n << ": got " << is_less_res << ", expected " << is_less_exp << ::std::endl;
-        EXPECT_TRUE(is_less_res == is_less_exp,
-                    "wrong effect from lex_compare Test 4.2: S1 < S2 (FIRST) && len(S1) > len(S2)");
+        EXPECT_EQ(is_less_exp, is_less_res, "wrong effect from lex_compare Test 4.2: S1 < S2 (FIRST) && len(S1) > len(S2)");
         {
             *host_keys.get() = 555;
             host_keys.update_data();
         }
 
         // CHECK 5.1: S1 > S2 (FIRST ELEMENT) && len(S1) == len(S2)
-        is_less_res = ::std::lexicographical_compare(make_new_policy<new_kernel_name<Policy, 9>>(exec), first1, last1, first2,
+        is_less_res = std::lexicographical_compare(CLONE_TEST_POLICY_IDX(exec, 9), first1, last1, first2,
                                                    last2, comp);
         wait_and_throw(exec);
 
-        if (is_less_res != 0)
-            ::std::cout << "N=" << n << ": got " << is_less_res << ", expected 0" << ::std::endl;
-        EXPECT_TRUE(is_less_res == 0, "wrong effect from lex_compare Test 5.1: S1 > S2 (FIRST) && len(S1) == len(S2)");
+        EXPECT_EQ(0, is_less_res, "wrong effect from lex_compare Test 5.1: S1 > S2 (FIRST) && len(S1) == len(S2)");
 
         // CHECK 5.2: S1 > S2 (FIRST ELEMENT) && len(S1) < len(S2)
-        is_less_res = ::std::lexicographical_compare(make_new_policy<new_kernel_name<Policy, 10>>(exec), first1, last1 - 1,
+        is_less_res = std::lexicographical_compare(CLONE_TEST_POLICY_IDX(exec, 10), first1, last1 - 1,
                                                    first2, last2, comp);
         wait_and_throw(exec);
 
         is_less_exp = n > 1 ? 0 : 1;
-        if (is_less_res != is_less_exp)
-            ::std::cout << "N=" << n << ": got " << is_less_res << ", expected " << is_less_exp << ::std::endl;
-        EXPECT_TRUE(is_less_res == is_less_exp,
-                    "wrong effect from lex_compare Test 5.2: S1 > S2 (FIRST) && len(S1) < len(S2)");
+        EXPECT_EQ(is_less_exp, is_less_res, "wrong effect from lex_compare Test 5.2: S1 > S2 (FIRST) && len(S1) < len(S2)");
     }
 };
 
