@@ -33,6 +33,26 @@ get_size(Size n)
     return n + a_size + b_size + c_size + d_size;
 }
 
+template <typename T>
+struct TransformOp
+{
+    T
+    operator()(T x) const
+    {
+        return x * 2;
+    }
+};
+
+template <typename IteratorValueType>
+struct IsMultipleOf3And2
+{
+    bool
+    operator()(IteratorValueType value) const
+    {
+        return (value % 3 == 0) && (value % 2 == 0);
+    }
+};
+
 DEFINE_TEST(test_remove)
 {
     DEFINE_TEST_CONSTRUCTOR(test_remove, 2.0f, 0.65f)
@@ -48,7 +68,7 @@ DEFINE_TEST(test_remove)
         host_keys.update_data();
 
         auto pos = (last - first) / 2;
-        auto res1 = ::std::remove(make_new_policy<new_kernel_name<Policy, 0>>(exec), first, last, T1(222 + pos));
+        auto res1 = std::remove(CLONE_TEST_POLICY_IDX(exec, 0), first, last, T1(222 + pos));
         wait_and_throw(exec);
 
         EXPECT_TRUE(res1 == last - 1, "wrong result from remove");
@@ -60,9 +80,8 @@ DEFINE_TEST(test_remove)
             auto exp = i + 222;
             if (i >= pos)
                 ++exp;
-            if (host_first1[i] != exp)
-                ::std::cout << "Error_1: i = " << i << ", expected " << exp << ", got " << host_first1[i] << ::std::endl;
-            EXPECT_TRUE(host_first1[i] == exp, "wrong effect from remove");
+
+            EXPECT_EQ(exp, host_first1[i], "wrong effect from remove");
         }
     }
 };
@@ -71,6 +90,16 @@ DEFINE_TEST(test_remove_if)
 {
     DEFINE_TEST_CONSTRUCTOR(test_remove_if, 2.0f, 0.65f)
 
+    template <typename T1, typename Size>
+    struct CheckState
+    {
+        Size pos;
+        bool operator()(T1 x) const
+        {
+            return x == T1(222 + pos);
+        }
+    };
+
     template <typename Policy, typename Iterator, typename Size>
     void
     operator()(Policy&& exec, Iterator first, Iterator last, Size n)
@@ -83,8 +112,7 @@ DEFINE_TEST(test_remove_if)
         host_keys.update_data();
 
         auto pos = (last - first) / 2;
-        auto res1 = ::std::remove_if(make_new_policy<new_kernel_name<Policy, 0>>(exec), first, last,
-                                   [=](T1 x) { return x == T1(222 + pos); });
+        auto res1 = std::remove_if(CLONE_TEST_POLICY_IDX(exec, 0), first, last, CheckState<T1, decltype(pos)>{pos});
         wait_and_throw(exec);
 
         EXPECT_TRUE(res1 == last - 1, "wrong result from remove_if");
@@ -96,11 +124,8 @@ DEFINE_TEST(test_remove_if)
             auto exp = i + 222;
             if (i >= pos)
                 ++exp;
-            if (host_first1[i] != exp)
-            {
-                ::std::cout << "Error_1: i = " << i << ", expected " << exp << ", got " << host_first1[i] << ::std::endl;
-            }
-            EXPECT_TRUE(host_first1[i] == exp, "wrong effect from remove_if");
+
+            EXPECT_EQ(exp, host_first1[i], "wrong effect from remove_if");
         }
     }
 };
@@ -123,8 +148,7 @@ DEFINE_TEST(test_unique)
         host_keys.update_data();
 
         // invoke
-        auto f = [](IteratorValueType a, IteratorValueType b) { return a == b; };
-        auto result_last = ::std::unique(make_new_policy<new_kernel_name<Policy, 0>>(exec), first, last, f);
+        auto result_last = std::unique(CLONE_TEST_POLICY_IDX(exec, 0), first, last, TestUtils::IsEqual<IteratorValueType>{});
         wait_and_throw(exec);
 
         auto result_size = result_last - first;
@@ -132,25 +156,13 @@ DEFINE_TEST(test_unique)
         std::int64_t expected_size = (n - 1) / 4 + 1;
 
         // check
-        bool is_correct = result_size == expected_size;
-#if _ONEDPL_DEBUG_SYCL
-        if (!is_correct)
-            ::std::cout << "buffer size: got " << result_last - first << ", expected " << expected_size << ::std::endl;
-#endif // _ONEDPL_DEBUG_SYCL
+        EXPECT_EQ(expected_size, result_size, "wrong effect from unique : incorrect size");
 
         host_keys.retrieve_data();
         auto host_first1 = host_keys.get();
-        for (int i = 0; i < ::std::min(result_size, expected_size) && is_correct; ++i)
+        for (int i = 0; i < std::min(result_size, expected_size); ++i)
         {
-            if (*(host_first1 + i) != i + 1)
-            {
-                is_correct = false;
-#if _ONEDPL_DEBUG_SYCL
-                ::std::cout << "got: " << *(host_first1 + i) << "[" << i << "], "
-                          << "expected: " << i + 1 << "[" << i << "]" << ::std::endl;
-#endif // _ONEDPL_DEBUG_SYCL
-            }
-            EXPECT_TRUE(is_correct, "wrong effect from unique");
+            EXPECT_EQ(i + 1, *(host_first1 + i), "wrong effect from unique : incorrect data");
         }
     }
 };
@@ -172,8 +184,8 @@ DEFINE_TEST(test_partition)
         host_keys.update_data();
 
         // invoke partition
-        auto unary_op = [](IteratorValueType value) { return (value % 3 == 0) && (value % 2 == 0); };
-        auto res = ::std::partition(make_new_policy<new_kernel_name<Policy, 0>>(exec), first, last, unary_op);
+        auto unary_op = IsMultipleOf3And2<IteratorValueType>{};
+        auto res = std::partition(CLONE_TEST_POLICY_IDX(exec, 0), first, last, unary_op);
         wait_and_throw(exec);
 
         // check
@@ -186,7 +198,7 @@ DEFINE_TEST(test_partition)
         host_keys.update_data();
 
         // invoke stable_partition
-        res = ::std::stable_partition(make_new_policy<new_kernel_name<Policy, 1>>(exec), first, last, unary_op);
+        res = std::stable_partition(CLONE_TEST_POLICY_IDX(exec, 1), first, last, unary_op);
         wait_and_throw(exec);
 
         host_keys.retrieve_data();
@@ -215,9 +227,7 @@ DEFINE_TEST(test_transform_inclusive_scan)
         ::std::fill(host_keys.get(), host_keys.get() + n, T1(1));
         host_keys.update_data();
 
-        auto res1 = ::std::transform_inclusive_scan(
-            make_new_policy<new_kernel_name<Policy, 0>>(exec), first1, last1, first2, ::std::plus<T1>(),
-            [](T1 x) { return x * 2; }, value);
+        auto res1 = std::transform_inclusive_scan(CLONE_TEST_POLICY_IDX(exec, 0), first1, last1, first2, std::plus<T1>(), TransformOp<T1>{}, value);
         wait_and_throw(exec);
 
         EXPECT_TRUE(res1 == last2, "wrong result from transform_inclusive_scan_1");
@@ -228,17 +238,12 @@ DEFINE_TEST(test_transform_inclusive_scan)
         for (int i = 0; i < last2 - first2; ++i)
         {
             ii += 2 * host_keys.get()[i];
-            if (host_vals.get()[i] != ii)
-            {
-                ::std::cout << "Error in scan_1: i = " << i << ", expected " << ii << ", got " << host_vals.get()[i]
-                            << ::std::endl;
-            }
-            EXPECT_TRUE(host_vals.get()[i] == ii, "wrong effect from transform_inclusive_scan_1");
+
+            EXPECT_EQ(ii, host_vals.get()[i], "wrong effect from transform_inclusive_scan_1");
         }
 
         // without initial value
-        auto res2 = ::std::transform_inclusive_scan(make_new_policy<new_kernel_name<Policy, 1>>(exec), first1, last1,
-                                                    first2, ::std::plus<T1>(), [](T1 x) { return x * 2; });
+        auto res2 = std::transform_inclusive_scan(CLONE_TEST_POLICY_IDX(exec, 1), first1, last1, first2, std::plus<T1>(), TransformOp<T1>{});
         EXPECT_TRUE(res2 == last2, "wrong result from transform_inclusive_scan_2");
 
         retrieve_data(host_keys, host_vals);
@@ -247,12 +252,8 @@ DEFINE_TEST(test_transform_inclusive_scan)
         for (int i = 0; i < last2 - first2; ++i)
         {
             ii += 2 * host_keys.get()[i];
-            if (host_vals.get()[i] != ii)
-            {
-                ::std::cout << "Error in scan_2: i = " << i << ", expected " << ii << ", got " << host_vals.get()[i]
-                            << ::std::endl;
-            }
-            EXPECT_TRUE(host_vals.get()[i] == ii, "wrong effect from transform_inclusive_scan_2");
+
+            EXPECT_EQ(ii, host_vals.get()[i], "wrong effect from transform_inclusive_scan_2");
         }
     }
 };
@@ -273,9 +274,7 @@ DEFINE_TEST(test_transform_exclusive_scan)
         ::std::fill(host_keys.get(), host_keys.get() + n, T1(1));
         host_keys.update_data();
 
-        auto res1 =
-            ::std::transform_exclusive_scan(make_new_policy<new_kernel_name<Policy, 2>>(exec), first1, last1, first2,
-                                          T1{}, ::std::plus<T1>(), [](T1 x) { return x * 2; });
+        auto res1 = std::transform_exclusive_scan(CLONE_TEST_POLICY_IDX(exec, 2), first1, last1, first2, T1{}, std::plus<T1>(), TransformOp<T1>{});
         wait_and_throw(exec);
 
         EXPECT_TRUE(res1 == last2, "wrong result from transform_exclusive_scan");
@@ -286,10 +285,8 @@ DEFINE_TEST(test_transform_exclusive_scan)
 
         for (size_t i = 0; i < last2 - first2; ++i)
         {
-            if (host_vals.get()[i] != ii)
-                ::std::cout << "Error: i = " << i << ", expected " << ii << ", got " << host_vals.get()[i] << ::std::endl;
+            EXPECT_EQ(ii, host_vals.get()[i], "wrong effect from transform_exclusive_scan : incorrect data");
 
-            //EXPECT_TRUE(host_vals.get()[i] == ii, "wrong effect from transform_exclusive_scan");
             ii += 2 * host_keys.get()[i];
         }
     }
@@ -311,8 +308,7 @@ DEFINE_TEST(test_copy_if)
         ::std::iota(host_keys.get(), host_keys.get() + n, T1(222));
         host_keys.update_data();
 
-        auto res1 = ::std::copy_if(make_new_policy<new_kernel_name<Policy, 0>>(exec), first1, last1, first2,
-                                   [](T1 x) { return x > -1; });
+        auto res1 = std::copy_if(CLONE_TEST_POLICY_IDX(exec, 0), first1, last1, first2, TestUtils::IsGreatThan<T1>{-1});
         wait_and_throw(exec);
 
         EXPECT_TRUE(res1 == last2, "wrong result from copy_if_1");
@@ -322,15 +318,11 @@ DEFINE_TEST(test_copy_if)
         for (int i = 0; i < res1 - first2; ++i)
         {
             auto exp = i + 222;
-            if (host_first2[i] != exp)
-            {
-                ::std::cout << "Error_1: i = " << i << ", expected " << exp << ", got " << host_first2[i] << ::std::endl;
-            }
-            EXPECT_TRUE(host_first2[i] == exp, "wrong effect from copy_if_1");
+
+            EXPECT_EQ(exp, host_first2[i], "wrong effect from copy_if_1 : incorrect data");
         }
 
-        auto res2 = ::std::copy_if(make_new_policy<new_kernel_name<Policy, 1>>(exec), first1, last1, first2,
-                                 [](T1 x) { return x % 2 == 1; });
+        auto res2 = std::copy_if(CLONE_TEST_POLICY_IDX(exec, 1), first1, last1, first2, TestUtils::IsOdd<T1>{});
         wait_and_throw(exec);
 
         EXPECT_TRUE(res2 == first2 + (last2 - first2) / 2, "wrong result from copy_if_2");
@@ -340,11 +332,8 @@ DEFINE_TEST(test_copy_if)
         for (int i = 0; i < res2 - first2; ++i)
         {
             auto exp = 2 * i + 1 + 222;
-            if (host_first2[i] != exp)
-            {
-                ::std::cout << "Error_2: i = " << i << ", expected " << exp << ", got " << host_first2[i] << ::std::endl;
-            }
-            EXPECT_TRUE(host_first2[i] == exp, "wrong effect from copy_if_2");
+
+            EXPECT_EQ(exp, host_first2[i], "wrong effect from copy_if_2 : incorrect data");
         }
     }
 };
@@ -369,10 +358,9 @@ DEFINE_TEST(test_unique_copy)
         update_data(host_keys, host_vals);
 
         // invoke
-        auto f = [](Iterator1ValueType a, Iterator1ValueType b) { return a == b; };
         auto result_first = first2;
-        auto result_last =
-            ::std::unique_copy(make_new_policy<new_kernel_name<Policy, 0>>(exec), first1, last1, result_first, f);
+        auto result_last = std::unique_copy(CLONE_TEST_POLICY_IDX(exec, 0), first1, last1,
+                                            result_first, TestUtils::IsEqual<Iterator1ValueType>{});
         wait_and_throw(exec);
 
         auto result_size = result_last - result_first;
@@ -380,24 +368,13 @@ DEFINE_TEST(test_unique_copy)
         std::int64_t expected_size = (n - 1) / 4 + 1;
 
         // check
-        bool is_correct = result_size == expected_size;
-#if _ONEDPL_DEBUG_SYCL
-        if (!is_correct)
-            ::std::cout << "buffer size: got " << result_last - result_first << ", expected " << expected_size
-                      << ::std::endl;
-#endif // _ONEDPL_DEBUG_SYCL
+        EXPECT_EQ(expected_size, result_size, "wrong effect from unique_copy : incorrect size");
 
         host_vals.retrieve_data();
         auto host_first2 = host_vals.get();
-        for (int i = 0; i < ::std::min(result_size, expected_size) && is_correct; ++i)
+        for (int i = 0; i < std::min(result_size, expected_size); ++i)
         {
-            if (*(host_first2 + i) != i + 1)
-            {
-                is_correct = false;
-                ::std::cout << "got: " << *(host_first2 + i) << "[" << i << "], "
-                          << "expected: " << i + 1 << "[" << i << "]" << ::std::endl;
-            }
-            EXPECT_TRUE(is_correct, "wrong effect from unique_copy");
+            EXPECT_EQ(i + 1, *(host_first2 + i), "wrong effect from unique_copy : incorrect data");
         }
     }
 };
@@ -418,7 +395,7 @@ DEFINE_TEST(test_partition_copy)
         using Iterator1ValueType = typename ::std::iterator_traits<Iterator1>::value_type;
         using Iterator2ValueType = typename ::std::iterator_traits<Iterator2>::value_type;
         using Iterator3ValueType = typename ::std::iterator_traits<Iterator3>::value_type;
-        auto f = [](Iterator1ValueType value) { return (value % 3 == 0) && (value % 2 == 0); };
+        auto f = IsMultipleOf3And2<Iterator1ValueType>{};
 
         // init
         ::std::iota(host_keys.get(), host_keys.get() + n, Iterator1ValueType{0});
@@ -428,7 +405,7 @@ DEFINE_TEST(test_partition_copy)
 
         // invoke
         auto res =
-            ::std::partition_copy(make_new_policy<new_kernel_name<Policy, 0>>(exec), first1, last1, first2, first3, f);
+            std::partition_copy(CLONE_TEST_POLICY_IDX(exec, 0), first1, last1, first2, first3, f);
         wait_and_throw(exec);
 
         retrieve_data(host_keys, host_vals, host_res);
@@ -440,43 +417,21 @@ DEFINE_TEST(test_partition_copy)
         auto exp_false_first = exp_false.begin();
 
         // invoke for expected
-        auto exp = ::std::partition_copy(host_keys.get(), host_keys.get() + n, exp_true_first, exp_false_first, f);
+        auto exp = std::partition_copy(host_keys.get(), host_keys.get() + n, exp_true_first, exp_false_first, f);
 
         // check
-        bool is_correct = (exp.first - exp_true_first) == (res.first - first2) &&
-                          (exp.second - exp_false_first) == (res.second - first3);
-#if _ONEDPL_DEBUG_SYCL
-        if (!is_correct)
-            ::std::cout << "N =" << n << ::std::endl
-                      << "buffer size: got {" << res.first - first2 << "," << res.second - first3 << "}, expected {"
-                      << exp.first - exp_true_first << "," << exp.second - exp_false_first << "}" << ::std::endl;
-#endif // _ONEDPL_DEBUG_SYCL
+        EXPECT_EQ(exp.first - exp_true_first, res.first - first2, "wrong effect from partition_copy : incorrect result #1");
+        EXPECT_EQ(exp.second - exp_false_first, res.second - first3, "wrong effect from partition_copy : incorrect result #2");
 
-        for (int i = 0; i < ::std::min(exp.first - exp_true_first, res.first - first2) && is_correct; ++i)
+        for (int i = 0; i < std::min(exp.first - exp_true_first, res.first - first2); ++i)
         {
-            if (*(exp_true_first + i) != *(host_vals.get() + i))
-            {
-                is_correct = false;
-#if _ONEDPL_DEBUG_SYCL
-                ::std::cout << "TRUE> got: " << *(host_vals.get() + i) << "[" << i << "], "
-                          << "expected: " << *(exp_true_first + i) << "[" << i << "]" << ::std::endl;
-#endif // _ONEDPL_DEBUG_SYCL
-            }
+            EXPECT_EQ(*(exp_true_first + i), *(host_vals.get() + i), "wrong effect from partition_copy : incorrect data #1");
         }
 
-        for (int i = 0; i < ::std::min(exp.second - exp_false_first, res.second - first3) && is_correct; ++i)
+        for (int i = 0; i < std::min(exp.second - exp_false_first, res.second - first3); ++i)
         {
-            if (*(exp_false_first + i) != *(host_res.get() + i))
-            {
-                is_correct = false;
-#if _ONEDPL_DEBUG_SYCL
-                ::std::cout << "FALSE> got: " << *(host_res.get() + i) << "[" << i << "], "
-                          << "expected: " << *(exp_false_first + i) << "[" << i << "]" << ::std::endl;
-#endif // _ONEDPL_DEBUG_SYCL
-            }
+            EXPECT_EQ(*(exp_false_first + i), *(host_res.get() + i), "wrong effect from partition_copy : incorrect data #2");
         }
-
-        EXPECT_TRUE(is_correct, "wrong effect from partition_copy");
     }
 };
 
@@ -501,7 +456,7 @@ DEFINE_TEST(test_set_intersection)
         host_keys.update_data(a_size);
         host_vals.update_data(b_size);
 
-        last3 = ::std::set_intersection(make_new_policy<new_kernel_name<Policy, 0>>(exec), first1, last1, first2, last2,
+        last3 = std::set_intersection(CLONE_TEST_POLICY_IDX(exec, 0), first1, last1, first2, last2,
                                       first3);
         wait_and_throw(exec);
 
@@ -524,7 +479,7 @@ DEFINE_TEST(test_set_intersection)
             host_keys.update_data(a_size);
             host_vals.update_data(b_size);
 
-            last3 = ::std::set_intersection(make_new_policy<new_kernel_name<Policy, 1>>(exec), first1, last1, first2,
+            last3 = std::set_intersection(CLONE_TEST_POLICY_IDX(exec, 1), first1, last1, first2,
                                           last2, first3);
             wait_and_throw(exec);
 
@@ -555,7 +510,7 @@ DEFINE_TEST(test_set_difference)
         host_keys.update_data(a_size);
         host_vals.update_data(b_size);
 
-        last3 = ::std::set_difference(make_new_policy<new_kernel_name<Policy, 0>>(exec), first1, last1, first2, last2, first3);
+        last3 = std::set_difference(CLONE_TEST_POLICY_IDX(exec, 0), first1, last1, first2, last2, first3);
         wait_and_throw(exec);
 
         int res_expect[a_size];
@@ -586,7 +541,7 @@ DEFINE_TEST(test_set_union)
         host_keys.update_data(a_size);
         host_vals.update_data(b_size);
 
-        last3 = ::std::set_union(make_new_policy<new_kernel_name<Policy, 0>>(exec), first1, last1, first2, last2, first3);
+        last3 = std::set_union(CLONE_TEST_POLICY_IDX(exec, 0), first1, last1, first2, last2, first3);
         wait_and_throw(exec);
 
         int res_expect[a_size + b_size];
@@ -618,7 +573,7 @@ DEFINE_TEST(test_set_symmetric_difference)
         host_keys.update_data(a_size);
         host_vals.update_data(b_size);
 
-        last3 = ::std::set_symmetric_difference(make_new_policy<new_kernel_name<Policy, 0>>(exec), first1, last1,
+        last3 = std::set_symmetric_difference(CLONE_TEST_POLICY_IDX(exec, 0), first1, last1,
                                                 first2, last2, first3);
         wait_and_throw(exec);
 
