@@ -44,7 +44,7 @@ struct test_replace_copy
         ::std::fill_n(out_first, n, trash);
         // Run replace_copy
         ::std::replace_copy(first, last, expected_first, old_value, new_value);
-        auto k = ::std::replace_copy(exec, first, last, out_first, old_value, new_value);
+        auto k = std::replace_copy(std::forward<Policy>(exec), first, last, out_first, old_value, new_value);
         EXPECT_EQ_N(expected_first, out_first, n, "wrong replace_copy effect");
         EXPECT_TRUE(out_last == k, "wrong return value from replace_copy");
     }
@@ -65,7 +65,7 @@ struct test_replace_copy_if
         ::std::fill_n(out_first, n, trash);
         // Run replace_copy_if
         replace_copy_if(first, last, expected_first, pred, new_value);
-        auto k = replace_copy_if(exec, first, last, out_first, pred, new_value);
+        auto k = replace_copy_if(std::forward<Policy>(exec), first, last, out_first, pred, new_value);
         EXPECT_EQ_N(expected_first, out_first, n, "wrong replace_copy_if effect");
         EXPECT_TRUE(out_last == k, "wrong return value from replace_copy_if");
     }
@@ -104,12 +104,8 @@ struct test_non_const
     void
     operator()(Policy&& exec, InputIterator input_iter, OutputInterator out_iter)
     {
-        auto is_even = [&](float64_t v) {
-            std::uint32_t i = (std::uint32_t)v;
-            return i % 2 == 0;
-        };
-
-        replace_copy_if(exec, input_iter, input_iter, out_iter, non_const(is_even), T(0));
+        auto is_even = TestUtils::IsEven<float64_t>{};
+        replace_copy_if(std::forward<Policy>(exec), input_iter, input_iter, out_iter, non_const(is_even), T(0));
     }
 };
 
