@@ -36,7 +36,7 @@ struct test_count
     operator()(Policy&& exec, Iterator first, Iterator last, T needle)
     {
         auto expected = ::std::count(first, last, needle);
-        auto result = ::std::count(exec, first, last, needle);
+        auto result = std::count(std::forward<Policy>(exec), first, last, needle);
         EXPECT_EQ(expected, result, "wrong count result");
     }
 };
@@ -49,18 +49,18 @@ struct test_count_if
     operator()(Policy&& exec, Iterator first, Iterator last, Predicate pred)
     {
         auto expected = ::std::count_if(first, last, pred);
-        auto result = ::std::count_if(exec, first, last, pred);
+        auto result = std::count_if(std::forward<Policy>(exec), first, last, pred);
         EXPECT_EQ(expected, result, "wrong count_if result");
     }
 };
 
 template <typename T>
-class IsEqual
+class IsEqualPred
 {
     T value;
 
   public:
-    IsEqual(T value_, OddTag) : value(value_) {}
+    IsEqualPred(T value_, OddTag) : value(value_) {}
     bool
     operator()(const T& x) const
     {
@@ -96,22 +96,18 @@ struct test_non_const
     void
     operator()(Policy&& exec, Iterator iter)
     {
-        auto is_even = [&](float64_t v) {
-            std::uint32_t i = (std::uint32_t)v;
-            return i % 2 == 0;
-        };
-        count_if(exec, iter, iter, non_const(is_even));
+        count_if(std::forward<Policy>(exec), iter, iter, non_const(TestUtils::IsEven<float64_t>{}));
     }
 };
 
 int
 main()
 {
-    test<std::int16_t>(42, IsEqual<std::int16_t>(50, OddTag()), [](std::int16_t j) { return j; });
+    test<std::int16_t>(42, IsEqualPred<std::int16_t>(50, OddTag()), [](std::int16_t j) { return j; });
     test<std::int32_t>(42, [](const std::int32_t&) { return true; }, [](std::int32_t j) { return j; });
-    test<float64_t>(42, IsEqual<float64_t>(50, OddTag()), [](std::int32_t j) { return float64_t(j); });
+    test<float64_t>(42, IsEqualPred<float64_t>(50, OddTag()), [](std::int32_t j) { return float64_t(j); });
 #if !TEST_DPCPP_BACKEND_PRESENT
-    test<Number>(Number(42, OddTag()), IsEqual<Number>(Number(50, OddTag()), OddTag()),
+    test<Number>(Number(42, OddTag()), IsEqualPred<Number>(Number(50, OddTag()), OddTag()),
                  [](std::int32_t j) { return Number(j, OddTag()); });
 #endif
 #ifdef _PSTL_TEST_COUNT_IF
