@@ -20,6 +20,7 @@
 #include <oneapi/dpl/iterator>
 
 #include "support/utils.h"
+#include "support/utils_invoke.h" // CLONE_TEST_POLICY_IDX
 #include "support/binary_search_utils.h"
 
 #include <cmath>
@@ -54,7 +55,7 @@ DEFINE_TEST(test_lower_bound)
     ::std::enable_if_t<oneapi::dpl::__internal::__is_hetero_execution_policy_v<::std::decay_t<Policy>> &&
                        is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator3>>
     operator()(Policy&& exec, Iterator1 first, Iterator1 last, Iterator2 value_first, Iterator2 value_last,
-               Iterator3 result_first, Iterator3 result_last, Size n)
+               Iterator3 result_first, Iterator3 /*result_last*/, Size n)
     {
         TestDataTransfer<UDTKind::eKeys, Size> host_keys(*this, n);
         TestDataTransfer<UDTKind::eVals, Size> host_vals(*this, n);
@@ -66,22 +67,19 @@ DEFINE_TEST(test_lower_bound)
         initialize_data(host_keys.get(), host_vals.get(), host_vals.get(), n);
         update_data(host_keys, host_vals, host_res);
 
-        auto new_policy = make_new_policy<new_kernel_name<Policy, 0>>(exec);
-        auto res1 = oneapi::dpl::lower_bound(new_policy, first, last, value_first, value_last, result_first);
+        auto res1 = oneapi::dpl::lower_bound( CLONE_TEST_POLICY_IDX(exec, 0), first, last, value_first, value_last, result_first);
         exec.queue().wait_and_throw();
 
-        EXPECT_TRUE(std::distance(result_first, res1) == n, "wrong return value, device policy");
+        EXPECT_EQ(n, std::distance(result_first, res1), "wrong return value, device policy");
         retrieve_data(host_vals, host_res);
         check_and_clean(host_res.get(), host_vals.get(), n);
         update_data(host_vals, host_res);
 
         // call algorithm with comparator
-        auto new_policy2 = make_new_policy<new_kernel_name<Policy, 1>>(exec);
-        auto res2 = oneapi::dpl::lower_bound(new_policy2, first, last, value_first, value_last, result_first,
-                                             [](ValueT first, ValueT second) { return first < second; });
+        auto res2 = oneapi::dpl::lower_bound(CLONE_TEST_POLICY_IDX(exec, 1), first, last, value_first, value_last, result_first, TestUtils::IsLess<ValueT>{});
         exec.queue().wait_and_throw();
 
-        EXPECT_TRUE(std::distance(result_first, res2) == n, "wrong return value, with predicate, device policy");
+        EXPECT_EQ(n, std::distance(result_first, res2), "wrong return value, with predicate, device policy");
         retrieve_data(host_vals, host_res);
         check_and_clean(host_res.get(), host_vals.get(), n);
     }
@@ -95,28 +93,26 @@ DEFINE_TEST(test_lower_bound)
 #endif
             is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator3>>
     operator()(Policy&& exec, Iterator1 first, Iterator1 last, Iterator2 value_first, Iterator2 value_last,
-               Iterator3 result_first, Iterator3 result_last, Size n)
+               Iterator3 result_first, Iterator3 /*result_last*/, Size n)
     {
         typedef typename ::std::iterator_traits<Iterator1>::value_type ValueT;
         // call algorithm with no optional arguments
         initialize_data(first, value_first, result_first, n);
 
-        auto res1 = oneapi::dpl::lower_bound(exec, first, last, value_first, value_last, result_first);
-        EXPECT_TRUE(std::distance(result_first, res1) == n, "wrong return value, host policy");
+        auto res1 = oneapi::dpl::lower_bound(CLONE_TEST_POLICY(exec), first, last, value_first, value_last, result_first);
+        EXPECT_EQ(n, std::distance(result_first, res1), "wrong return value, host policy");
         check_and_clean(result_first, value_first, n);
 
         // call algorithm with comparator
-        auto res2 = oneapi::dpl::lower_bound(exec, first, last, value_first, value_last, result_first,
-                                             [](ValueT first, ValueT second) { return first < second; });
-        EXPECT_TRUE(std::distance(result_first, res2) == n, "wrong return value, with predicate, host policy");
+        auto res2 = oneapi::dpl::lower_bound(CLONE_TEST_POLICY(exec), first, last, value_first, value_last, result_first, TestUtils::IsLess<ValueT>{});
+        EXPECT_EQ(n, std::distance(result_first, res2), "wrong return value, with predicate, host policy");
         check_and_clean(result_first, value_first, n);
     }
 
     // specialization for non-random_access iterators
     template <typename Policy, typename Iterator1, typename Iterator2, typename Iterator3, typename Size>
     ::std::enable_if_t<!is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator3>>
-    operator()(Policy&& exec, Iterator1 first, Iterator1 last, Iterator2 value_first, Iterator2 value_last,
-               Iterator3 result_first, Iterator3 result_last, Size n)
+    operator()(Policy&&, Iterator1, Iterator1, Iterator2, Iterator2, Iterator3, Iterator3, Size)
     {
     }
 };
