@@ -53,8 +53,8 @@ struct test_is_heap
     {
         using namespace std;
         bool expected = is_heap(first, last);
-        bool actual = is_heap(exec, first, last);
-        EXPECT_TRUE(expected == actual, "wrong return value from is_heap");
+        bool actual = is_heap(std::forward<Policy>(exec), first, last);
+        EXPECT_EQ(expected, actual, "wrong return value from is_heap");
     }
 
     // is_heap works only with random access iterators
@@ -74,8 +74,8 @@ struct test_is_heap_predicate
     {
         using namespace std;
         bool expected = is_heap(first, last, pred);
-        bool actual = is_heap(exec, first, last, pred);
-        EXPECT_TRUE(expected == actual, "wrong return value from is_heap with predicate");
+        bool actual = is_heap(std::forward<Policy>(exec), first, last, pred);
+        EXPECT_EQ(expected, actual, "wrong return value from is_heap with predicate");
     }
 
     // is_heap works only with random access iterators
@@ -95,8 +95,8 @@ struct test_is_heap_until
     {
         using namespace std;
         Iterator expected = is_heap_until(first, last);
-        Iterator actual = is_heap_until(exec, first, last);
-        EXPECT_TRUE(expected == actual, "wrong return value from is_heap_until");
+        Iterator actual = is_heap_until(std::forward<Policy>(exec), first, last);
+        EXPECT_EQ(expected, actual, "wrong return value from is_heap_until");
     }
 
     // is_heap, is_heap_until works only with random access iterators
@@ -116,8 +116,8 @@ struct test_is_heap_until_predicate
     {
         using namespace std;
         const Iterator expected = is_heap_until(first, last, pred);
-        const Iterator actual = is_heap_until(exec, first, last, pred);
-        EXPECT_TRUE(expected == actual, "wrong return value from is_heap_until with predicate");
+        const Iterator actual = is_heap_until(std::forward<Policy>(exec), first, last, pred);
+        EXPECT_EQ(expected, actual, "wrong return value from is_heap_until with predicate");
     }
 
     // is_heap, is_heap_until works only with random access iterators
@@ -193,7 +193,7 @@ struct test_non_const_is_heap
     void
     operator()(Policy&& exec, Iterator iter)
     {
-        is_heap(exec, iter, iter, non_const(::std::less<T>()));
+        is_heap(std::forward<Policy>(exec), iter, iter, non_const(std::less<T>()));
     }
 };
 
@@ -204,7 +204,7 @@ struct test_non_const_is_heap_until
     void
     operator()(Policy&& exec, Iterator iter)
     {
-        is_heap_until(exec, iter, iter, non_const(::std::less<T>()));
+        is_heap_until(std::forward<Policy>(exec), iter, iter, non_const(std::less<T>()));
     }
 };
 
