@@ -23,19 +23,19 @@ main()
 
     // should always pick the "offset" device since executed inline
     // there is no overlap and so "offset" is always unloaded at selection time
-    auto f = [u](int i) { return u[0]; };
+    auto f = [u](int) { return u[0]; };
 
     constexpr bool just_call_submit = false;
     constexpr bool call_select_before_submit = true;
 
-    test_initialization<policy_t, int>(u);
-    test_select<policy_t, decltype(u), decltype(f)&, false>(u, f);
-    test_submit_and_wait_on_event<just_call_submit, policy_t>(u, f);
-    test_submit_and_wait_on_event<call_select_before_submit, policy_t>(u, f);
-    test_submit_and_wait<just_call_submit, policy_t>(u, f);
-    test_submit_and_wait<call_select_before_submit, policy_t>(u, f);
-    test_submit_and_wait_on_group<just_call_submit, policy_t>(u, f);
-    test_submit_and_wait_on_group<call_select_before_submit, policy_t>(u, f);
+    EXPECT_EQ(0, (test_initialization<policy_t, int>(u)), "");
+    EXPECT_EQ(0, (test_select<policy_t, decltype(u), decltype(f)&, false>(u, f)), "");
+    EXPECT_EQ(0, (test_submit_and_wait_on_event<just_call_submit, policy_t>(u, f)), "");
+    EXPECT_EQ(0, (test_submit_and_wait_on_event<call_select_before_submit, policy_t>(u, f)), "");
+    EXPECT_EQ(0, (test_submit_and_wait<just_call_submit, policy_t>(u, f)), "");
+    EXPECT_EQ(0, (test_submit_and_wait<call_select_before_submit, policy_t>(u, f)), "");
+    EXPECT_EQ(0, (test_submit_and_wait_on_group<just_call_submit, policy_t>(u, f)), "");
+    EXPECT_EQ(0, (test_submit_and_wait_on_group<call_select_before_submit, policy_t>(u, f)), "");
 
     return TestUtils::done();
 }
