@@ -23,50 +23,63 @@ DEFINE_TEST_PERM_IT(test_sort, PermItIndexTag)
 {
     DEFINE_TEST_PERM_IT_CONSTRUCTOR(test_sort, 2.0f, 0.65f)
 
-    template <typename TIterator, typename Size>
-    void generate_data(TIterator itBegin, TIterator itEnd, Size n)
+    template <typename Size>
+    struct TestImplementation
     {
-        Size index = 0;
-        for (auto it = itBegin; it != itEnd; ++it, ++index)
-            *it = n - index;
-    }
+        Size n;
+        TestDataTransfer<UDTKind::eKeys, Size>& host_keys;
 
-    template <typename TIterator>
-    void check_results(TIterator itBegin, TIterator itEnd)
-    {
-        const auto result = std::is_sorted(itBegin, itEnd);
-        EXPECT_TRUE(result, "Wrong sort data results");
-    }
+        template <typename Policy, typename TPermutationIterator>
+        void
+        operator()(Policy&& exec, TPermutationIterator permItBegin, TPermutationIterator permItEnd) const
+        {
+            const auto host_keys_ptr = host_keys.get();
+
+            const auto testing_n = permItEnd - permItBegin;
+
+            // Fill full source data set (not only values iterated by permutation iterator)
+            generate_data(host_keys_ptr, host_keys_ptr + n, n);
+            host_keys.update_data();
+
+            dpl::sort(CLONE_TEST_POLICY(exec), permItBegin, permItEnd);
+            wait_and_throw(exec);
+
+            // Copy data back
+            std::vector<TestValueType> resultTest(testing_n);
+            dpl::copy(CLONE_TEST_POLICY(exec), permItBegin, permItEnd, resultTest.begin());
+            wait_and_throw(exec);
+
+            // Check results
+            check_results(resultTest.begin(), resultTest.end());
+        }
+
+        template <typename TIterator>
+        void generate_data(TIterator itBegin, TIterator itEnd, Size n) const
+        {
+            Size index = 0;
+            for (auto it = itBegin; it != itEnd; ++it, (void) ++index)
+                *it = n - index;
+        }
+
+        template <typename TIterator>
+        void
+        check_results(TIterator itBegin, TIterator itEnd) const
+        {
+            const auto result = std::is_sorted(itBegin, itEnd);
+            EXPECT_TRUE(result, "Wrong sort data results");
+        }
+    };
 
     template <typename Policy, typename Iterator1, typename Size>
     void
-    operator()(Policy&& exec, Iterator1 first1, Iterator1 last1, Size n)
+    operator()(Policy&& exec, Iterator1 first1, Iterator1 /*last1*/, Size n)
     {
         if constexpr (is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator1>)
         {
             TestDataTransfer<UDTKind::eKeys, Size> host_keys(*this, n);     // sorting data
-            const auto host_keys_ptr = host_keys.get();
 
             test_through_permutation_iterator<Iterator1, Size, PermItIndexTag>{first1, n}(
-                [&](auto permItBegin, auto permItEnd)
-                {
-                    const auto testing_n = permItEnd - permItBegin;
-
-                    // Fill full source data set (not only values iterated by permutation iterator)
-                    generate_data(host_keys_ptr, host_keys_ptr + n, n);
-                    host_keys.update_data();
-
-                    dpl::sort(exec, permItBegin, permItEnd);
-                    wait_and_throw(exec);
-
-                    // Copy data back
-                    std::vector<TestValueType> resultTest(testing_n);
-                    dpl::copy(exec, permItBegin, permItEnd, resultTest.begin());
-                    wait_and_throw(exec);
-
-                    // Check results
-                    check_results(resultTest.begin(), resultTest.end());
-                });
+                std::forward<Policy>(exec), TestImplementation<Size>{n, host_keys});
         }
     }
 };
