@@ -20,6 +20,7 @@
 #include "oneapi/dpl/iterator"
 
 #include "support/utils.h"
+#include "support/utils_invoke.h" // CLONE_TEST_POLICY_IDX
 #include "support/scan_serial_impl.h"
 
 #if TEST_DPCPP_BACKEND_PRESENT
@@ -112,14 +113,13 @@ DEFINE_TEST_2(test_exclusive_scan_by_segment, BinaryPredicate, BinaryOperation)
     template <typename Policy, typename Iterator1, typename Iterator2, typename Iterator3, typename Size>
     ::std::enable_if_t<oneapi::dpl::__internal::__is_hetero_execution_policy_v<::std::decay_t<Policy>> &&
                        is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator3>>
-    operator()(Policy&& exec, Iterator1 keys_first, Iterator1 keys_last, Iterator2 vals_first, Iterator2 vals_last,
-               Iterator3 val_res_first, Iterator3 val_res_last, Size n)
+    operator()(Policy&& exec, Iterator1 keys_first, Iterator1 keys_last, Iterator2 vals_first, Iterator2 /*vals_last*/,
+               Iterator3 val_res_first, Iterator3 /*val_res_last*/, Size n)
     {
         TestDataTransfer<UDTKind::eKeys, Size> host_keys(*this, n);
         TestDataTransfer<UDTKind::eVals, Size> host_vals(*this, n);
         TestDataTransfer<UDTKind::eRes, Size> host_val_res(*this, n);
 
-        typedef typename ::std::iterator_traits<Iterator1>::value_type KeyT;
         typedef typename ::std::iterator_traits<Iterator2>::value_type ValT;
 
         const ValT zero = 0;
@@ -129,12 +129,11 @@ DEFINE_TEST_2(test_exclusive_scan_by_segment, BinaryPredicate, BinaryOperation)
         initialize_data(host_keys.get(), host_vals.get(), host_val_res.get(), n);
         update_data(host_keys, host_vals, host_val_res);
 
-        auto new_policy = make_new_policy<new_kernel_name<Policy, 0>>(exec);
         auto res1 =
-            oneapi::dpl::exclusive_scan_by_segment(new_policy, keys_first, keys_last, vals_first, val_res_first);
+            oneapi::dpl::exclusive_scan_by_segment(CLONE_TEST_POLICY_IDX(exec, 0), keys_first, keys_last, vals_first, val_res_first);
         exec.queue().wait_and_throw();
 
-        EXPECT_TRUE(std::distance(val_res_first, res1) == n, "wrong return value, device policy");
+        EXPECT_EQ(n, std::distance(val_res_first, res1), "wrong return value, device policy");
         retrieve_data(host_vals, host_val_res);
         check_values(host_keys.get(), host_vals.get(), host_val_res.get(), n, zero);
 
@@ -142,12 +141,11 @@ DEFINE_TEST_2(test_exclusive_scan_by_segment, BinaryPredicate, BinaryOperation)
         initialize_data(host_keys.get(), host_vals.get(), host_val_res.get(), n);
         update_data(host_keys, host_vals, host_val_res);
 
-        auto new_policy2 = make_new_policy<new_kernel_name<Policy, 1>>(exec);
         auto res2 =
-            oneapi::dpl::exclusive_scan_by_segment(new_policy2, keys_first, keys_last, vals_first, val_res_first, init);
+            oneapi::dpl::exclusive_scan_by_segment(CLONE_TEST_POLICY_IDX(exec, 1), keys_first, keys_last, vals_first, val_res_first, init);
         exec.queue().wait_and_throw();
 
-        EXPECT_TRUE(std::distance(val_res_first, res2) == n, "wrong return value, init, device policy");
+        EXPECT_EQ(n, std::distance(val_res_first, res2), "wrong return value, init, device policy");
         retrieve_data(host_vals, host_val_res);
         check_values(host_keys.get(), host_vals.get(), host_val_res.get(), n, init);
 
@@ -155,13 +153,11 @@ DEFINE_TEST_2(test_exclusive_scan_by_segment, BinaryPredicate, BinaryOperation)
         initialize_data(host_keys.get(), host_vals.get(), host_val_res.get(), n);
         update_data(host_keys, host_vals, host_val_res);
 
-        auto binary_op = [](ValT first, ValT second) { return first + second; };
-        auto new_policy3 = make_new_policy<new_kernel_name<Policy, 2>>(exec);
-        auto res3 = oneapi::dpl::exclusive_scan_by_segment(new_policy3, keys_first, keys_last, vals_first,
+        auto res3 = oneapi::dpl::exclusive_scan_by_segment(CLONE_TEST_POLICY_IDX(exec, 2), keys_first, keys_last, vals_first,
                                                            val_res_first, init, BinaryPredicate());
         exec.queue().wait_and_throw();
 
-        EXPECT_TRUE(std::distance(val_res_first, res3) == n, "wrong return value, init and predicate, device policy");
+        EXPECT_EQ(n, std::distance(val_res_first, res3), "wrong return value, init and predicate, device policy");
         retrieve_data(host_vals, host_val_res);
         check_values(host_keys.get(), host_vals.get(), host_val_res.get(), n, init, BinaryPredicate());
 
@@ -169,12 +165,11 @@ DEFINE_TEST_2(test_exclusive_scan_by_segment, BinaryPredicate, BinaryOperation)
         initialize_data(host_keys.get(), host_vals.get(), host_val_res.get(), n);
         update_data(host_keys, host_vals, host_val_res);
 
-        auto new_policy4 = make_new_policy<new_kernel_name<Policy, 3>>(exec);
-        auto res4 = oneapi::dpl::exclusive_scan_by_segment(new_policy4, keys_first, keys_last, vals_first,
+        auto res4 = oneapi::dpl::exclusive_scan_by_segment(CLONE_TEST_POLICY_IDX(exec, 3), keys_first, keys_last, vals_first,
                                                            val_res_first, init, BinaryPredicate(), BinaryOperation());
         exec.queue().wait_and_throw();
 
-        EXPECT_TRUE(std::distance(val_res_first, res4) == n, "wrong return value, init and predicate and operator, device policy");
+        EXPECT_EQ(n, std::distance(val_res_first, res4), "wrong return value, init and predicate and operator, device policy");
         retrieve_data(host_vals, host_val_res);
         check_values(host_keys.get(), host_vals.get(), host_val_res.get(), n, init, BinaryPredicate(),
                      BinaryOperation());
@@ -188,8 +183,8 @@ DEFINE_TEST_2(test_exclusive_scan_by_segment, BinaryPredicate, BinaryOperation)
         !oneapi::dpl::__internal::__is_hetero_execution_policy_v<::std::decay_t<Policy>> &&
 #endif // TEST_DPCPP_BACKEND_PRESENT
             is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator3>>
-    operator()(Policy&& exec, Iterator1 keys_first, Iterator1 keys_last, Iterator2 vals_first, Iterator2 vals_last,
-               Iterator3 val_res_first, Iterator3 val_res_last, Size n)
+    operator()(Policy&& exec, Iterator1 keys_first, Iterator1 keys_last, Iterator2 vals_first, Iterator2 /*vals_last*/,
+               Iterator3 val_res_first, Iterator3 /*val_res_last*/, Size n)
     {
         typedef typename ::std::iterator_traits<Iterator2>::value_type ValT;
 
@@ -198,37 +193,36 @@ DEFINE_TEST_2(test_exclusive_scan_by_segment, BinaryPredicate, BinaryOperation)
 
         // call algorithm with no optional arguments
         initialize_data(keys_first, vals_first, val_res_first, n);
-        auto res1 = oneapi::dpl::exclusive_scan_by_segment(exec, keys_first, keys_last, vals_first, val_res_first);
-        EXPECT_TRUE(std::distance(val_res_first, res1) == n, "wrong return value, host policy");
+        auto res1 = oneapi::dpl::exclusive_scan_by_segment(CLONE_TEST_POLICY(exec), keys_first, keys_last, vals_first, val_res_first);
+        EXPECT_EQ(n, std::distance(val_res_first, res1), "wrong return value, host policy");
         check_values(keys_first, vals_first, val_res_first, n, zero);
 
         // call algorithm with init
         initialize_data(keys_first, vals_first, val_res_first, n);
         auto res2 =
-            oneapi::dpl::exclusive_scan_by_segment(exec, keys_first, keys_last, vals_first, val_res_first, init);
-        EXPECT_TRUE(std::distance(val_res_first, res2) == n, "wrong return value, init, host policy");
+            oneapi::dpl::exclusive_scan_by_segment(CLONE_TEST_POLICY(exec), keys_first, keys_last, vals_first, val_res_first, init);
+        EXPECT_EQ(n, std::distance(val_res_first, res2), "wrong return value, init, host policy");
         check_values(keys_first, vals_first, val_res_first, n, init);
 
         // call algorithm with init and predicate
         initialize_data(keys_first, vals_first, val_res_first, n);
-        auto res3 = oneapi::dpl::exclusive_scan_by_segment(exec, keys_first, keys_last, vals_first, val_res_first, init,
+        auto res3 = oneapi::dpl::exclusive_scan_by_segment(CLONE_TEST_POLICY(exec), keys_first, keys_last, vals_first, val_res_first, init,
                                                            BinaryPredicate());
-        EXPECT_TRUE(std::distance(val_res_first, res3) == n, "wrong return value, init and predicate, host policy");
+        EXPECT_EQ(n, std::distance(val_res_first, res3), "wrong return value, init and predicate, host policy");
         check_values(keys_first, vals_first, val_res_first, n, init, BinaryPredicate());
 
         // call algorithm with init, predicate, and operator
         initialize_data(keys_first, vals_first, val_res_first, n);
-        auto res4 = oneapi::dpl::exclusive_scan_by_segment(exec, keys_first, keys_last, vals_first, val_res_first, init,
+        auto res4 = oneapi::dpl::exclusive_scan_by_segment(CLONE_TEST_POLICY(exec), keys_first, keys_last, vals_first, val_res_first, init,
                                                            BinaryPredicate(), BinaryOperation());
-        EXPECT_TRUE(std::distance(val_res_first, res4) == n, "wrong return value, init and predicate and operator, host policy");
+        EXPECT_EQ(n, std::distance(val_res_first, res4), "wrong return value, init and predicate and operator, host policy");
         check_values(keys_first, vals_first, val_res_first, n, init, BinaryPredicate(), BinaryOperation());
     }
 
     // specialization for non-random_access iterators
     template <typename Policy, typename Iterator1, typename Iterator2, typename Iterator3, typename Size>
     ::std::enable_if_t<!is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator3>>
-    operator()(Policy&& exec, Iterator1 keys_first, Iterator1 keys_last, Iterator2 vals_first, Iterator2 vals_last,
-               Iterator3 val_res_first, Iterator3 val_res_last, Size n)
+    operator()(Policy&&, Iterator1, Iterator1, Iterator2, Iterator2, Iterator3, Iterator3, Size)
     {
     }
 };
