@@ -19,6 +19,7 @@
 #include _PSTL_TEST_HEADER(numeric)
 
 #include "support/utils.h"
+#include "support/utils_device_copyable.h"
 
 using namespace TestUtils;
 
@@ -29,7 +30,7 @@ struct test_long_reduce
     void
     operator()(Policy&& exec, Iterator first, Iterator last, T init, BinaryOp binary, T expected)
     {
-        T result_r = ::std::reduce(exec, first, last, init, binary);
+        T result_r = std::reduce(std::forward<Policy>(exec), first, last, std::move(init), binary);
         EXPECT_EQ(expected, result_r, "bad result from reduce(exec, first, last, init, binary_op)");
     }
 };
@@ -53,6 +54,15 @@ test_long_form(T init, BinaryOp binary_op, F f)
 
         invoke_on_all_policies<0>()(test_long_reduce<T>(), in.begin(), in.end(), init, binary_op, expected);
         invoke_on_all_policies<1>()(test_long_reduce<T>(), in.cbegin(), in.cend(), init, binary_op, expected);
+        if constexpr (std::is_same_v<BinaryOp, std::plus<T>>)
+        {
+            invoke_on_all_policies<2>()(test_long_reduce<T>(), in.begin(), in.end(), NoDefaultCtorWrapper<T>{init},
+                                        std::plus<NoDefaultCtorWrapper<T>>{}, NoDefaultCtorWrapper<T>{expected});
+            // Test with MoveOnlyWrapper on host policies
+            iterator_invoker<std::random_access_iterator_tag, /*reverse_iterator=*/std::false_type>()(
+                oneapi::dpl::execution::par_unseq, test_long_reduce<T>(), in.begin(), in.end(),
+                MoveOnlyWrapper<T>{init}, std::plus<MoveOnlyWrapper<T>>{}, MoveOnlyWrapper<T>{expected});
+        }
     }
 }
 
@@ -64,8 +74,8 @@ struct test_short_reduce
     {
         using namespace std;
 
-        Sum r0 = init + reduce(exec, first, last);
-        EXPECT_EQ(expected, r0, "bad result from reduce(exec, first, last)");
+        Sum r0 = init + reduce(std::forward<Policy>(exec), first, last);
+        EXPECT_EQ(expected, r0, "bad result from reduce(std::forward<Policy>(exec), first, last)");
     }
 };
 
@@ -77,8 +87,8 @@ struct test_short_reduce_init
     {
         using namespace std;
 
-        Sum r1 = reduce(exec, first, last, init);
-        EXPECT_EQ(expected, r1, "bad result from reduce(exec, first, last, init)");
+        Sum r1 = reduce(std::forward<Policy>(exec), first, last, init);
+        EXPECT_EQ(expected, r1, "bad result from reduce(std::forward<Policy>(exec), first, last, init)");
     }
 };
 
