@@ -49,8 +49,8 @@ DEFINE_TEST_2(test_reduce_by_segment, BinaryPredicate, BinaryOperation)
 {
     DEFINE_TEST_CONSTRUCTOR(test_reduce_by_segment, 1.0f, 1.0f)
 
-    template <typename Iterator1, typename Iterator2, typename Iterator3, typename Iterator4, typename Size>
-    void initialize_data(Iterator1 host_keys, Iterator2 host_vals, Iterator3 host_key_res, Iterator4 host_val_res,
+    template <typename Iterator1, typename Iterator2, typename Iterator3, typename Size>
+    void initialize_data(Iterator1 host_keys, Iterator2 host_vals, Iterator3 host_val_res,
                          Size n)
     {
         // T keys[n] = { 1, 2, 3, 4, 1, 1, 2, 2, 3, 3, 4, 4, 1, 1, 1, ...};
@@ -139,8 +139,8 @@ DEFINE_TEST_2(test_reduce_by_segment, BinaryPredicate, BinaryOperation)
     ::std::enable_if_t<oneapi::dpl::__internal::__is_hetero_execution_policy_v<::std::decay_t<Policy>> &&
                        is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator3> &&
                        is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator4>>
-    operator()(Policy&& exec, Iterator1 keys_first, Iterator1 keys_last, Iterator2 vals_first, Iterator2 vals_last,
-               Iterator3 key_res_first, Iterator3 key_res_last, Iterator4 val_res_first, Iterator4 val_res_last, Size n)
+    operator()(Policy&& exec, Iterator1 keys_first, Iterator1 keys_last, Iterator2 vals_first, Iterator2 /*vals_last*/,
+               Iterator3 key_res_first, Iterator3 /*key_res_last*/, Iterator4 val_res_first, Iterator4 /*val_res_last*/, Size n)
     {
         TestDataTransfer<UDTKind::eKeys, Size> host_keys(*this, n);
         TestDataTransfer<UDTKind::eVals, Size> host_vals(*this, n);
@@ -150,23 +150,23 @@ DEFINE_TEST_2(test_reduce_by_segment, BinaryPredicate, BinaryOperation)
         typedef typename ::std::iterator_traits<Iterator1>::value_type KeyT;
         typedef typename ::std::iterator_traits<Iterator2>::value_type ValT;
 
-        initialize_data(host_keys.get(), host_vals.get(), host_res_keys.get(), host_res.get(), n);
+        initialize_data(host_keys.get(), host_vals.get(), host_res.get(), n);
         update_data(host_keys, host_vals, host_res_keys, host_res);
 
         std::pair<Iterator3, Iterator4> res;
         if constexpr (std::is_same_v<std::equal_to<KeyT>, std::decay_t<BinaryPredicate>> &&
                       std::is_same_v<std::plus<ValT>, std::decay_t<BinaryOperation>>)
         {
-            res = oneapi::dpl::reduce_by_segment(exec, keys_first, keys_last, vals_first, key_res_first, val_res_first);
+            res = oneapi::dpl::reduce_by_segment(CLONE_TEST_POLICY(exec), keys_first, keys_last, vals_first, key_res_first, val_res_first);
         }
         else if constexpr (std::is_same_v<std::plus<ValT>, std::decay_t<BinaryOperation>>)
         {
-            res = oneapi::dpl::reduce_by_segment(exec, keys_first, keys_last, vals_first, key_res_first, val_res_first,
+            res = oneapi::dpl::reduce_by_segment(CLONE_TEST_POLICY(exec), keys_first, keys_last, vals_first, key_res_first, val_res_first,
                                                  BinaryPredicate());
         }
         else
         {
-            res = oneapi::dpl::reduce_by_segment(exec, keys_first, keys_last, vals_first, key_res_first, val_res_first,
+            res = oneapi::dpl::reduce_by_segment(CLONE_TEST_POLICY(exec), keys_first, keys_last, vals_first, key_res_first, val_res_first,
                                                  BinaryPredicate(), BinaryOperation());
         }
         exec.queue().wait_and_throw();
@@ -188,13 +188,13 @@ DEFINE_TEST_2(test_reduce_by_segment, BinaryPredicate, BinaryOperation)
 #endif
             is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator3> &&
             is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator4>>
-    operator()(Policy&& exec, Iterator1 keys_first, Iterator1 keys_last, Iterator2 vals_first, Iterator2 vals_last,
-               Iterator3 key_res_first, Iterator3 key_res_last, Iterator4 val_res_first, Iterator4 val_res_last, Size n)
+    operator()(Policy&& exec, Iterator1 keys_first, Iterator1 keys_last, Iterator2 vals_first, Iterator2 /*vals_last*/,
+               Iterator3 key_res_first, Iterator3 /*key_res_last*/, Iterator4 val_res_first, Iterator4 /*val_res_last*/, Size n)
     {
         typedef typename ::std::iterator_traits<Iterator1>::value_type KeyT;
         typedef typename ::std::iterator_traits<Iterator2>::value_type ValT;
 
-        initialize_data(keys_first, vals_first, key_res_first, val_res_first, n);
+        initialize_data(keys_first, vals_first, val_res_first, n);
 
         std::pair<Iterator3, Iterator4> res;
         if constexpr (std::is_same_v<std::equal_to<KeyT>, std::decay_t<BinaryPredicate>> &&
@@ -224,19 +224,26 @@ DEFINE_TEST_2(test_reduce_by_segment, BinaryPredicate, BinaryOperation)
               typename Size>
     ::std::enable_if_t<!is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator3> ||
                            !is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator4>>
-    operator()(Policy&& exec, Iterator1 keys_first, Iterator1 keys_last, Iterator2 vals_first, Iterator2 vals_last,
-               Iterator3 key_res_first, Iterator3 key_res_last, Iterator4 val_res_first, Iterator4 val_res_last, Size n)
+    operator()(Policy&&, Iterator1, Iterator1, Iterator2, Iterator2, Iterator3, Iterator3, Iterator4, Iterator4, Size)
     {
     }
 };
 
-#if TEST_DPCPP_BACKEND_PRESENT
-template <sycl::usm::alloc alloc_type, typename KernelName, typename T>
-void
-test_flag_pred()
+struct FlagPred
 {
-    sycl::queue q = TestUtils::get_test_queue();
+    template <typename T>
+    bool operator()(const T& a, const T& b) const
+    {
+        using KeyT = std::decay_t<decltype(b)>;
+        return b != KeyT(1);
+    }
+};
 
+#if TEST_DPCPP_BACKEND_PRESENT
+template <sycl::usm::alloc alloc_type, typename KernelName, typename T, typename Policy>
+void
+test_flag_pred(Policy&& exec)
+{
     // Initialize data
     //T keys[n1] = { 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0 };
     //T vals[n1] = { 1, 2, 3, 4, 1, 2, 3, 4, 1, 2, 3, 4, 1, 2 };
@@ -244,7 +251,7 @@ test_flag_pred()
     // keys_result = {1, 1, 1};
     // vals_result = {11, 12, 10};
 
-    auto prepare_data = [](int n, T* key_head, T* val_head, T* key_res_head, T* val_res_head)
+    auto prepare_data = [](int n, T* key_head, T* val_head)
         {
             for (int i = 0; i < n; ++i)
             {
@@ -259,26 +266,26 @@ test_flag_pred()
     T key_res_head_on_host[n] = {};
     T val_res_head_on_host[n] = {};
 
-    prepare_data(n, key_head_on_host, val_head_on_host, key_res_head_on_host, val_res_head_on_host);
-    auto flag_pred = [](const auto& a, const auto& b) {
-        using KeyT = ::std::decay_t<decltype(b)>;
-        return b != KeyT(1);
-    };
+    prepare_data(n, key_head_on_host, val_head_on_host);
+    FlagPred flag_pred;
     // allocate USM memory and copying data to USM shared/device memory
-    TestUtils::usm_data_transfer<alloc_type, T> dt_helper1(q, std::begin(key_head_on_host),     std::end(key_head_on_host));
-    TestUtils::usm_data_transfer<alloc_type, T> dt_helper2(q, std::begin(val_head_on_host),     std::end(val_head_on_host));
-    TestUtils::usm_data_transfer<alloc_type, T> dt_helper3(q, std::begin(key_res_head_on_host), std::end(key_res_head_on_host));
-    TestUtils::usm_data_transfer<alloc_type, T> dt_helper4(q, std::begin(val_res_head_on_host), std::end(val_res_head_on_host));
+    TestUtils::usm_data_transfer<alloc_type, T> dt_helper1(exec, std::begin(key_head_on_host),     std::end(key_head_on_host));
+    TestUtils::usm_data_transfer<alloc_type, T> dt_helper2(exec, std::begin(val_head_on_host),     std::end(val_head_on_host));
+    TestUtils::usm_data_transfer<alloc_type, T> dt_helper3(exec, std::begin(key_res_head_on_host), std::end(key_res_head_on_host));
+    TestUtils::usm_data_transfer<alloc_type, T> dt_helper4(exec, std::begin(val_res_head_on_host), std::end(val_res_head_on_host));
     auto key_head     = dt_helper1.get_data();
     auto val_head     = dt_helper2.get_data();
     auto key_res_head = dt_helper3.get_data();
     auto val_res_head = dt_helper4.get_data();
 
     // call algorithm
-    auto new_policy = oneapi::dpl::execution::make_device_policy<TestUtils::unique_kernel_name<
-        TestUtils::unique_kernel_name<KernelName, 1>, TestUtils::uniq_kernel_index<alloc_type>()>>(q);
-    auto res1 =
-        oneapi::dpl::reduce_by_segment(new_policy, key_head, key_head + n, val_head, key_res_head, val_res_head, flag_pred, std::plus<T>());
+    using _NewKernelName = TestUtils::unique_kernel_name<
+        TestUtils::unique_kernel_name<KernelName, 1>,
+        TestUtils::uniq_kernel_index<alloc_type>()>;
+
+    auto res1 = oneapi::dpl::reduce_by_segment(
+            CLONE_TEST_POLICY_NAME(exec, _NewKernelName),
+            key_head, key_head + n, val_head, key_res_head, val_res_head, flag_pred, std::plus<T>());
 
     //retrieve result on the host and check the result
     dt_helper3.retrieve_data(key_res_head_on_host);
@@ -341,6 +348,21 @@ run_test()
     run_test_on_device<use_device_alloc, ValueType, BinaryPredicate, BinaryOperation>();
 }
 
+#if TEST_DPCPP_BACKEND_PRESENT
+
+class KernelName1;
+class KernelName2;
+
+template <typename Policy>
+void test_flag_pred_impl(Policy&& exec)
+{
+    // test with flag pred
+    test_flag_pred<sycl::usm::alloc::device, KernelName1, std::uint64_t>(CLONE_TEST_POLICY(exec));
+    test_flag_pred<sycl::usm::alloc::device, KernelName2, MatrixPoint<float>>(CLONE_TEST_POLICY(exec));
+}
+
+#endif // TEST_DPCPP_BACKEND_PRESENT
+
 int
 main()
 {
@@ -352,9 +374,13 @@ main()
 #endif
 
 #if TEST_DPCPP_BACKEND_PRESENT
-    // test with flag pred
-    test_flag_pred<sycl::usm::alloc::device, class KernelName1, std::uint64_t>();
-    test_flag_pred<sycl::usm::alloc::device, class KernelName2, MatrixPoint<float>>();
+
+    auto policy = TestUtils::get_dpcpp_test_policy();
+    test_flag_pred_impl(policy);
+
+#if TEST_CHECK_COMPILATION_WITH_DIFF_POLICY_VAL_CATEGORY
+    TestUtils::check_compilation(policy, [](auto&& policy) { test_flag_pred_impl(std::forward<decltype(policy)>(policy)); });
+#endif
 #endif // TEST_DPCPP_BACKEND_PRESENT
 
 #if !_PSTL_RED_BY_SEG_WINDOWS_COMPILE_ORDER_BROKEN
