@@ -13,22 +13,24 @@
 //
 //===----------------------------------------------------------------------===//
 
-#include <oneapi/dpl/execution>
-
 #include "support/test_config.h"
 
+#include <oneapi/dpl/execution>
+
 #if _ENABLE_RANGES_TESTING
 #include <oneapi/dpl/ranges>
 #endif
 
 #include "support/utils.h"
+#include "support/utils_invoke.h" // for CLONE_TEST_POLICY macro
 
 #include <iostream>
 
-std::int32_t
-main()
-{
 #if _ENABLE_RANGES_TESTING
+template <typename Policy>
+void
+test_impl(Policy&& exec)
+{
     constexpr int max_n = 10;
     int data1[max_n]     = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9};
     int data2[max_n]     = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9};
@@ -43,19 +45,28 @@ main()
 
         auto view = views::all(A);
                           
-        auto exec = TestUtils::default_dpcpp_policy;
-        using Policy = decltype(exec);
-        auto exec1 = TestUtils::make_new_policy<TestUtils::new_kernel_name<Policy, 0>>(exec);
-        auto exec2 = TestUtils::make_new_policy<TestUtils::new_kernel_name<Policy, 1>>(exec);
-             
-        res1 = equal(exec1, view, B);
-        res2 = equal(exec2, C, view, ::std::equal_to<>{});
+        res1 = equal(CLONE_TEST_POLICY_IDX(exec, 0), view, B);
+        res2 = equal(CLONE_TEST_POLICY_IDX(exec, 1), C, view, std::equal_to<>{});
     }
 
     //check result
     EXPECT_TRUE(res1, "wrong result from equal with sycl ranges");
     EXPECT_FALSE(res2, "wrong result from equal with sycl ranges");
+}
+#endif // _ENABLE_RANGES_TESTING
 
+std::int32_t
+main()
+{
+#if _ENABLE_RANGES_TESTING
+
+    auto policy = TestUtils::get_dpcpp_test_policy();
+    test_impl(policy);
+
+#if TEST_CHECK_COMPILATION_WITH_DIFF_POLICY_VAL_CATEGORY
+    TestUtils::check_compilation(policy, [](auto&& policy) { test_impl(std::forward<decltype(policy)>(policy)); });
+#endif
 #endif //_ENABLE_RANGES_TESTING
+
     return TestUtils::done(_ENABLE_RANGES_TESTING);
 }
