@@ -22,10 +22,12 @@ main()
     using namespace test_std_ranges;
     namespace dpl_ranges = oneapi::dpl::ranges;
 
-    test_range_algo<0>{}(dpl_ranges::find_if_not, std::ranges::find_if_not, pred1);
-    test_range_algo<1>{}(dpl_ranges::find_if_not, std::ranges::find_if_not, pred, proj);
-    test_range_algo<2, P2>{}(dpl_ranges::find_if_not, std::ranges::find_if_not, pred, &P2::x);
-    test_range_algo<3, P2>{}(dpl_ranges::find_if_not, std::ranges::find_if_not, pred, &P2::proj);
+    auto find_if_not_checker = TEST_PREPARE_CALLABLE(std::ranges::find_if_not);
+
+    test_range_algo<0>{big_sz}(dpl_ranges::find_if_not, find_if_not_checker, pred1);
+    test_range_algo<1>{}(dpl_ranges::find_if_not, find_if_not_checker, pred, proj);
+    test_range_algo<2, P2>{}(dpl_ranges::find_if_not, find_if_not_checker, pred, &P2::x);
+    test_range_algo<3, P2>{}(dpl_ranges::find_if_not, find_if_not_checker, pred, &P2::proj);
 #endif //_ENABLE_STD_RANGES_TESTING
 
     return TestUtils::done(_ENABLE_STD_RANGES_TESTING);
