@@ -25,7 +25,7 @@ main()
     auto search_checker = TEST_PREPARE_CALLABLE(std::ranges::search);
 
     test_range_algo<0, int, data_in_in>{big_sz}(dpl_ranges::search,  search_checker, binary_pred);
-    test_range_algo<1, int, data_in_in>{}(dpl_ranges::search,  search_checker, binary_pred, proj, proj);
+    test_range_algo<1, int, data_in_in>{}(dpl_ranges::search,  search_checker, binary_pred, proj);
     test_range_algo<2, P2, data_in_in>{}(dpl_ranges::search,  search_checker, binary_pred, &P2::x, &P2::x);
     test_range_algo<3, P2, data_in_in>{}(dpl_ranges::search,  search_checker, binary_pred, &P2::proj, &P2::proj);
 #endif //_ENABLE_STD_RANGES_TESTING
