@@ -297,14 +297,14 @@ struct non_const_wrapper_tagged : non_const_wrapper
     ::std::enable_if_t<IsPositiveCondition == is_same_iterator_category_v<Iterator, IteratorTag>>
     operator()(Policy&& exec, Iterator iter)
     {
-        Op()(exec, iter);
+        Op()(std::forward<Policy>(exec), iter);
     }
 
     template <typename Policy, typename InputIterator, typename OutputIterator>
     ::std::enable_if_t<IsPositiveCondition == is_same_iterator_category_v<OutputIterator, IteratorTag>>
     operator()(Policy&& exec, InputIterator input_iter, OutputIterator out_iter)
     {
-        Op()(exec, input_iter, out_iter);
+        Op()(std::forward<Policy>(exec), input_iter, out_iter);
     }
 
     template <typename Policy, typename Iterator>
@@ -371,15 +371,16 @@ struct iterator_invoker
     operator()(Policy&& exec, Op op, Iterator begin, typename ::std::iterator_traits<Iterator>::difference_type n,
         Iterator expected, Rest&&... rest)
     {
-        op(exec, make_iterator<Iterator>()(begin), n, make_iterator<Iterator>()(expected), ::std::forward<Rest>(rest)...);
+        op(std::forward<Policy>(exec), make_iterator<Iterator>()(begin), n, make_iterator<Iterator>()(expected),
+           std::forward<Rest>(rest)...);
     }
 
     template <typename Policy, typename Op, typename Iterator, typename... Rest>
     ::std::enable_if_t<is_base_of_iterator_category<::std::random_access_iterator_tag, Iterator>::value>
     operator()(Policy&& exec, Op op, Iterator begin, typename ::std::iterator_traits<Iterator>::difference_type n, Rest&&... rest)
     {
-        invoke_if<Iterator>()(n <= sizeLimit<decltype(n)>, op, exec, make_iterator<Iterator>()(begin), n,
-                              ::std::forward<Rest>(rest)...);
+        invoke_if<Iterator>()(n <= sizeLimit<decltype(n)>, op, std::forward<Policy>(exec),
+                              make_iterator<Iterator>()(begin), n, std::forward<Rest>(rest)...);
     }
 
     template <typename Policy, typename Op, typename Iterator, typename... Rest>
@@ -387,9 +388,10 @@ struct iterator_invoker
                            !::std::is_base_of_v<non_const_wrapper, Op>>
     operator()(Policy&& exec, Op op, Iterator inputBegin, Iterator inputEnd, Rest&&... rest)
     {
-        invoke_if<Iterator>()(
-            std::distance(inputBegin, inputEnd) <= sizeLimit<decltype(std::distance(inputBegin, inputEnd))>, op, exec,
-            make_iterator<Iterator>()(inputBegin), make_iterator<Iterator>()(inputEnd), std::forward<Rest>(rest)...);
+        invoke_if<Iterator>()(std::distance(inputBegin, inputEnd) <=
+                                  sizeLimit<decltype(std::distance(inputBegin, inputEnd))>,
+                              op, std::forward<Policy>(exec), make_iterator<Iterator>()(inputBegin),
+                              make_iterator<Iterator>()(inputEnd), std::forward<Rest>(rest)...);
     }
 
     template <typename Policy, typename Op, typename InputIterator, typename OutputIterator, typename... Rest>
@@ -397,10 +399,11 @@ struct iterator_invoker
     operator()(Policy&& exec, Op op, InputIterator inputBegin, InputIterator inputEnd, OutputIterator outputBegin,
                Rest&&... rest)
     {
-        invoke_if<InputIterator>()(
-            std::distance(inputBegin, inputEnd) <= sizeLimit<decltype(std::distance(inputBegin, inputEnd))>, op, exec,
-            make_iterator<InputIterator>()(inputBegin), make_iterator<InputIterator>()(inputEnd),
-            make_iterator<OutputIterator>()(outputBegin), std::forward<Rest>(rest)...);
+        invoke_if<InputIterator>()(std::distance(inputBegin, inputEnd) <=
+                                       sizeLimit<decltype(std::distance(inputBegin, inputEnd))>,
+                                   op, std::forward<Policy>(exec), make_iterator<InputIterator>()(inputBegin),
+                                   make_iterator<InputIterator>()(inputEnd),
+                                   make_iterator<OutputIterator>()(outputBegin), std::forward<Rest>(rest)...);
     }
 
     template <typename Policy, typename Op, typename InputIterator, typename OutputIterator, typename... Rest>
@@ -409,10 +412,10 @@ struct iterator_invoker
                OutputIterator outputEnd, Rest&&... rest)
     {
         invoke_if<InputIterator>()(
-            std::distance(inputBegin, inputEnd) <= sizeLimit<decltype(std::distance(inputBegin, inputEnd))>, op, exec,
-            make_iterator<InputIterator>()(inputBegin), make_iterator<InputIterator>()(inputEnd),
-            make_iterator<OutputIterator>()(outputBegin), make_iterator<OutputIterator>()(outputEnd),
-            std::forward<Rest>(rest)...);
+            std::distance(inputBegin, inputEnd) <= sizeLimit<decltype(std::distance(inputBegin, inputEnd))>, op,
+            std::forward<Policy>(exec), make_iterator<InputIterator>()(inputBegin),
+            make_iterator<InputIterator>()(inputEnd), make_iterator<OutputIterator>()(outputBegin),
+            make_iterator<OutputIterator>()(outputEnd), std::forward<Rest>(rest)...);
     }
 
     template <typename Policy, typename Op, typename InputIterator1, typename InputIterator2, typename OutputIterator,
@@ -423,10 +426,10 @@ struct iterator_invoker
     {
         invoke_if<InputIterator1>()(
             std::distance(inputBegin1, inputEnd1) <= sizeLimit<decltype(std::distance(inputBegin1, inputEnd1))>, op,
-            exec, make_iterator<InputIterator1>()(inputBegin1), make_iterator<InputIterator1>()(inputEnd1),
-            make_iterator<InputIterator2>()(inputBegin2), make_iterator<InputIterator2>()(inputEnd2),
-            make_iterator<OutputIterator>()(outputBegin), make_iterator<OutputIterator>()(outputEnd),
-            std::forward<Rest>(rest)...);
+            std::forward<Policy>(exec), make_iterator<InputIterator1>()(inputBegin1),
+            make_iterator<InputIterator1>()(inputEnd1), make_iterator<InputIterator2>()(inputBegin2),
+            make_iterator<InputIterator2>()(inputEnd2), make_iterator<OutputIterator>()(outputBegin),
+            make_iterator<OutputIterator>()(outputEnd), std::forward<Rest>(rest)...);
     }
     
     template <typename Policy, typename Op, typename InputIterator1, typename InputIterator2, typename InputIterator3, typename OutputIterator,
@@ -438,11 +441,11 @@ struct iterator_invoker
     {
         invoke_if<InputIterator1>()(
             std::distance(inputBegin1, inputEnd1) <= sizeLimit<decltype(std::distance(inputBegin1, inputEnd1))>, op,
-            exec, make_iterator<InputIterator1>()(inputBegin1), make_iterator<InputIterator1>()(inputEnd1),
-            make_iterator<InputIterator2>()(inputBegin2), make_iterator<InputIterator2>()(inputEnd2),
-            make_iterator<InputIterator3>()(inputBegin3), make_iterator<InputIterator3>()(inputEnd3),
-            make_iterator<OutputIterator>()(outputBegin), make_iterator<OutputIterator>()(outputEnd),
-            std::forward<Rest>(rest)...);
+            std::forward<Policy>(exec), make_iterator<InputIterator1>()(inputBegin1),
+            make_iterator<InputIterator1>()(inputEnd1), make_iterator<InputIterator2>()(inputBegin2),
+            make_iterator<InputIterator2>()(inputEnd2), make_iterator<InputIterator3>()(inputBegin3),
+            make_iterator<InputIterator3>()(inputEnd3), make_iterator<OutputIterator>()(outputBegin),
+            make_iterator<OutputIterator>()(outputEnd), std::forward<Rest>(rest)...);
     }
 };
 
@@ -461,7 +464,7 @@ struct iterator_invoker<IteratorTag, /* IsReverse = */ ::std::true_type>
                          ::std::is_base_of_v<non_const_wrapper, Op>>
     operator()(Policy&& exec, Op op, Iterator iter)
     {
-        op(::std::forward<Policy>(exec), make_iterator<Iterator>()(iter));
+        op(std::forward<Policy>(exec), make_iterator<Iterator>()(iter));
     }
 
     // A version with 2 iterators which is used for non_const testcases
@@ -470,7 +473,7 @@ struct iterator_invoker<IteratorTag, /* IsReverse = */ ::std::true_type>
                          ::std::is_base_of_v<non_const_wrapper, Op>>
     operator()(Policy&& exec, Op op, InputIterator input_iter, OutputIterator out_iter)
     {
-        op(::std::forward<Policy>(exec), make_iterator<InputIterator>()(input_iter),
+        op(std::forward<Policy>(exec), make_iterator<InputIterator>()(input_iter),
            make_iterator<OutputIterator>()(out_iter));
     }
 
@@ -479,7 +482,7 @@ struct iterator_invoker<IteratorTag, /* IsReverse = */ ::std::true_type>
     operator()(Policy&& exec, Op op, Iterator begin, typename ::std::iterator_traits<Iterator>::difference_type n, Rest&&... rest)
     {
         if (n <= sizeLimit<decltype(n)>)
-            op(exec, make_iterator<Iterator>()(begin + n), n, ::std::forward<Rest>(rest)...);
+            op(std::forward<Policy>(exec), make_iterator<Iterator>()(begin + n), n, std::forward<Rest>(rest)...);
     }
 
     template <typename Policy, typename Op, typename Iterator, typename... Rest>
@@ -488,7 +491,8 @@ struct iterator_invoker<IteratorTag, /* IsReverse = */ ::std::true_type>
         Rest&&... rest)
     {
         if (n <= sizeLimit<decltype(n)>)
-            op(exec, make_iterator<Iterator>()(begin + n), n, make_iterator<Iterator>()(expected + n), ::std::forward<Rest>(rest)...);
+            op(std::forward<Policy>(exec), make_iterator<Iterator>()(begin + n), n,
+               make_iterator<Iterator>()(expected + n), std::forward<Rest>(rest)...);
     }
 
     template <typename Policy, typename Op, typename Iterator, typename... Rest>
@@ -496,7 +500,8 @@ struct iterator_invoker<IteratorTag, /* IsReverse = */ ::std::true_type>
     operator()(Policy&& exec, Op op, Iterator begin, typename ::std::iterator_traits<Iterator>::difference_type n, Iterator expected, Rest&&... rest)
     {
         if (n <= sizeLimit<decltype(n)>)
-            op(exec, make_iterator<Iterator>()(std::next(begin, n)), n, make_iterator<Iterator>()(std::next(expected, n)), ::std::forward<Rest>(rest)...);
+            op(std::forward<Policy>(exec), make_iterator<Iterator>()(std::next(begin, n)), n,
+               make_iterator<Iterator>()(std::next(expected, n)), std::forward<Rest>(rest)...);
     }
 
     template <typename Policy, typename Op, typename Iterator, typename... Rest>
@@ -505,7 +510,7 @@ struct iterator_invoker<IteratorTag, /* IsReverse = */ ::std::true_type>
     operator()(Policy&& exec, Op op, Iterator inputBegin, Iterator inputEnd, Rest&&... rest)
     {
         if (std::distance(inputBegin, inputEnd) <= sizeLimit<decltype(std::distance(inputBegin, inputEnd))>)
-            op(exec, make_iterator<Iterator>()(inputEnd), make_iterator<Iterator>()(inputBegin),
+            op(std::forward<Policy>(exec), make_iterator<Iterator>()(inputEnd), make_iterator<Iterator>()(inputBegin),
                std::forward<Rest>(rest)...);
     }
 
@@ -515,7 +520,8 @@ struct iterator_invoker<IteratorTag, /* IsReverse = */ ::std::true_type>
                Rest&&... rest)
     {
         if (std::distance(inputBegin, inputEnd) <= sizeLimit<decltype(std::distance(inputBegin, inputEnd))>)
-            op(exec, make_iterator<InputIterator>()(inputEnd), make_iterator<InputIterator>()(inputBegin),
+            op(std::forward<Policy>(exec), make_iterator<InputIterator>()(inputEnd),
+               make_iterator<InputIterator>()(inputBegin),
                make_iterator<OutputIterator>()(outputBegin + (inputEnd - inputBegin)), std::forward<Rest>(rest)...);
     }
 
@@ -525,9 +531,9 @@ struct iterator_invoker<IteratorTag, /* IsReverse = */ ::std::true_type>
                OutputIterator outputEnd, Rest&&... rest)
     {
         if (std::distance(inputBegin, inputEnd) <= sizeLimit<decltype(std::distance(inputBegin, inputEnd))>)
-            op(exec, make_iterator<InputIterator>()(inputEnd), make_iterator<InputIterator>()(inputBegin),
-               make_iterator<OutputIterator>()(outputEnd), make_iterator<OutputIterator>()(outputBegin),
-               std::forward<Rest>(rest)...);
+            op(std::forward<Policy>(exec), make_iterator<InputIterator>()(inputEnd),
+               make_iterator<InputIterator>()(inputBegin), make_iterator<OutputIterator>()(outputEnd),
+               make_iterator<OutputIterator>()(outputBegin), std::forward<Rest>(rest)...);
     }
 
     template <typename Policy, typename Op, typename InputIterator1, typename InputIterator2, typename OutputIterator,
@@ -537,10 +543,10 @@ struct iterator_invoker<IteratorTag, /* IsReverse = */ ::std::true_type>
                InputIterator2 inputEnd2, OutputIterator outputBegin, OutputIterator outputEnd, Rest&&... rest)
     {
         if (std::distance(inputBegin1, inputEnd1) <= sizeLimit<decltype(std::distance(inputBegin1, inputEnd1))>)
-            op(exec, make_iterator<InputIterator1>()(inputEnd1), make_iterator<InputIterator1>()(inputBegin1),
-               make_iterator<InputIterator2>()(inputEnd2), make_iterator<InputIterator2>()(inputBegin2),
-               make_iterator<OutputIterator>()(outputEnd), make_iterator<OutputIterator>()(outputBegin),
-               std::forward<Rest>(rest)...);
+            op(std::forward<Policy>(exec), make_iterator<InputIterator1>()(inputEnd1),
+               make_iterator<InputIterator1>()(inputBegin1), make_iterator<InputIterator2>()(inputEnd2),
+               make_iterator<InputIterator2>()(inputBegin2), make_iterator<OutputIterator>()(outputEnd),
+               make_iterator<OutputIterator>()(outputBegin), std::forward<Rest>(rest)...);
     }
 
     template <typename Policy, typename Op, typename InputIterator1, typename InputIterator2, typename InputIterator3,
@@ -551,11 +557,11 @@ struct iterator_invoker<IteratorTag, /* IsReverse = */ ::std::true_type>
                OutputIterator outputBegin, OutputIterator outputEnd, Rest&&... rest)
     {
         if (std::distance(inputBegin1, inputEnd1) <= sizeLimit<decltype(std::distance(inputBegin1, inputEnd1))>)
-            op(exec, make_iterator<InputIterator1>()(inputEnd1), make_iterator<InputIterator1>()(inputBegin1),
-               make_iterator<InputIterator2>()(inputEnd2), make_iterator<InputIterator2>()(inputBegin2),
-               make_iterator<InputIterator3>()(inputEnd3), make_iterator<InputIterator3>()(inputBegin3),
-               make_iterator<OutputIterator>()(outputEnd), make_iterator<OutputIterator>()(outputBegin),
-               std::forward<Rest>(rest)...);
+            op(std::forward<Policy>(exec), make_iterator<InputIterator1>()(inputEnd1),
+               make_iterator<InputIterator1>()(inputBegin1), make_iterator<InputIterator2>()(inputEnd2),
+               make_iterator<InputIterator2>()(inputBegin2), make_iterator<InputIterator3>()(inputEnd3),
+               make_iterator<InputIterator3>()(inputBegin3), make_iterator<OutputIterator>()(outputEnd),
+               make_iterator<OutputIterator>()(outputBegin), std::forward<Rest>(rest)...);
     }
 
 };
