@@ -13,15 +13,15 @@
 //
 //===----------------------------------------------------------------------===//
 
-#ifndef __TEST_SYCL_ALLOC_UTILS_H
-#define __TEST_SYCL_ALLOC_UTILS_H
+#ifndef _TEST_SYCL_ALLOC_UTILS_H
+#define _TEST_SYCL_ALLOC_UTILS_H
 
 #if TEST_DPCPP_BACKEND_PRESENT
 
 #include <list>
 #include <memory>
 
-#include "oneapi/dpl/pstl/hetero/dpcpp/sycl_defs.h"
+#include "utils_sycl_defs.h"
 
 namespace TestUtils
 {
@@ -157,7 +157,8 @@ public:
 
             //TODO: support copying data provided by non-contiguous iterator
             auto __src = std::addressof(*__it);
-            assert(std::addressof(*(__it + __objects_count)) - __src == __objects_count);
+            // we remove 1 from __objects_count to avoid of out-of-range iterator dereferencing
+            assert(std::addressof(*(__it + __objects_count - 1)) - __src == __objects_count - 1);
 
             copy_data_impl(__src, __ptr + __offset, __objects_count);
         }
@@ -182,7 +183,8 @@ public:
 
             //TODO: support copying data provided by non-contiguous iterator
             auto __dst = std::addressof(*__it);
-            assert(std::addressof(*(__it + __objects_count)) - __dst == __objects_count);
+            // we remove 1 from __objects_count to avoid of out-of-range iterator dereferencing
+            assert(std::addressof(*(__it + __objects_count - 1)) - __dst == __objects_count - 1);
 
             copy_data_impl(__ptr + __offset, __dst, __objects_count);
         }
@@ -192,7 +194,7 @@ private:
 
     void copy_data_impl(_ValueType* __src, _ValueType* __ptr, __difference_type __count)
     {
-#if __LIBSYCL_VERSION >= 50300
+#if TEST_LIBSYCL_VERSION >= 50300
         __queue.copy(__src, __ptr, __count);
 #else
         auto __p = __ptr;
@@ -203,7 +205,7 @@ private:
                 *(__p + __id) = *(__src + __id);
                 });
             });
-#endif // __LIBSYCL_VERSION >= 50300
+#endif // TEST_LIBSYCL_VERSION >= 50300
         __queue.wait();
     }
 
@@ -220,4 +222,4 @@ private:
 
 #endif // TEST_DPCPP_BACKEND_PRESENT
 
-#endif // __TEST_SYCL_ALLOC_UTILS_H
+#endif // _TEST_SYCL_ALLOC_UTILS_H
