@@ -30,11 +30,11 @@ class output_iterator
     friend class output_iterator;
 
   public:
-    typedef std::output_iterator_tag iterator_category;
+    typedef ::std::output_iterator_tag iterator_category;
     typedef void value_type;
-    typedef typename std::iterator_traits<It>::difference_type difference_type;
-    typedef It pointer;
-    typedef typename std::iterator_traits<It>::reference reference;
+    typedef typename ::std::iterator_traits<It>::difference_type difference_type;
+    typedef typename ::std::iterator_traits<It>::pointer pointer;
+    typedef typename ::std::iterator_traits<It>::reference reference;
 
     It
     base() const
@@ -69,45 +69,54 @@ class output_iterator
     void operator,(T const &) DELETE_FUNCTION;
 };
 
-template <class It, class ItTraits = It>
+template <class It>
 class input_iterator
 {
-    typedef std::iterator_traits<ItTraits> Traits;
     It it_;
 
-    template <class U, class T>
+    template <class U>
     friend class input_iterator;
 
   public:
-    typedef std::input_iterator_tag iterator_category;
-    typedef typename Traits::value_type value_type;
-    typedef typename Traits::difference_type difference_type;
-    typedef It pointer;
-    typedef typename Traits::reference reference;
+    typedef ::std::input_iterator_tag iterator_category;
+    typedef typename ::std::iterator_traits<It>::value_type value_type;
+    typedef typename ::std::iterator_traits<It>::difference_type difference_type;
+    typedef typename ::std::iterator_traits<It>::pointer pointer;
+    typedef typename ::std::iterator_traits<It>::reference reference;
 
-    It
+    constexpr It
     base() const
     {
         return it_;
     }
 
-    input_iterator() : it_() {}
-    explicit input_iterator(It it) : it_(it) {}
-    template <class U, class T>
-    input_iterator(const input_iterator<U, T>& u) : it_(u.it_)
+    constexpr input_iterator() : it_() {}
+    explicit constexpr input_iterator(It it) : it_(it) {}
+    template <class U>
+    constexpr input_iterator(const input_iterator<U>& u) : it_(u.it_)
     {
     }
 
-    reference operator*() const { return *it_; }
-    pointer operator->() const { return it_; }
+    constexpr reference operator*() const { return *it_; }
+    constexpr pointer operator->() const 
+    {
+        if constexpr (::std::is_pointer_v<It>)
+        {
+            return it_;
+        }
+        else
+        {
+            return it_.operator->();
+        }
+    }
 
-    input_iterator&
+    constexpr input_iterator&
     operator++()
     {
         ++it_;
         return *this;
     }
-    input_iterator
+    constexpr input_iterator
     operator++(int)
     {
         input_iterator tmp(*this);
@@ -115,12 +124,12 @@ class input_iterator
         return tmp;
     }
 
-    friend bool
+    friend constexpr bool
     operator==(const input_iterator& x, const input_iterator& y)
     {
         return x.it_ == y.it_;
     }
-    friend bool
+    friend constexpr bool
     operator!=(const input_iterator& x, const input_iterator& y)
     {
         return !(x == y);
@@ -130,16 +139,16 @@ class input_iterator
     void operator,(T const &) DELETE_FUNCTION;
 };
 
-template <class T, class TV, class U, class UV>
-inline bool
-operator==(const input_iterator<T, TV>& x, const input_iterator<U, UV>& y)
+template <class T, class U>
+inline bool constexpr
+operator==(const input_iterator<T>& x, const input_iterator<U>& y)
 {
     return x.base() == y.base();
 }
 
-template <class T, class TV, class U, class UV>
-inline bool
-operator!=(const input_iterator<T, TV>& x, const input_iterator<U, UV>& y)
+template <class T, class U>
+inline bool constexpr
+operator!=(const input_iterator<T>& x, const input_iterator<U>& y)
 {
     return !(x == y);
 }
@@ -153,35 +162,45 @@ class forward_iterator
     friend class forward_iterator;
 
   public:
-    typedef std::forward_iterator_tag iterator_category;
-    typedef typename std::iterator_traits<It>::value_type value_type;
-    typedef typename std::iterator_traits<It>::difference_type difference_type;
-    typedef It pointer;
-    typedef typename std::iterator_traits<It>::reference reference;
+    typedef ::std::forward_iterator_tag iterator_category;
+    typedef typename ::std::iterator_traits<It>::value_type value_type;
+    typedef typename ::std::iterator_traits<It>::difference_type difference_type;
+    typedef typename ::std::iterator_traits<It>::pointer pointer;
+    typedef typename ::std::iterator_traits<It>::reference reference;
 
-    It
+    constexpr It
     base() const
     {
         return it_;
     }
 
-    forward_iterator() : it_() {}
-    explicit forward_iterator(It it) : it_(it) {}
+    constexpr forward_iterator() : it_() {}
+    explicit constexpr forward_iterator(It it) : it_(it) {}
     template <class U>
-    forward_iterator(const forward_iterator<U>& u) : it_(u.it_)
+    constexpr forward_iterator(const forward_iterator<U>& u) : it_(u.it_)
     {
     }
 
-    reference operator*() const { return *it_; }
-    pointer operator->() const { return it_; }
+    constexpr reference operator*() const { return *it_; }
+    constexpr pointer operator->() const 
+    {
+        if constexpr (::std::is_pointer_v<It>)
+        {
+            return it_;
+        }
+        else
+        {
+            return it_.operator->();
+        }
+    }
 
-    forward_iterator&
+    constexpr forward_iterator&
     operator++()
     {
         ++it_;
         return *this;
     }
-    forward_iterator
+    constexpr forward_iterator
     operator++(int)
     {
         forward_iterator tmp(*this);
@@ -189,12 +208,12 @@ class forward_iterator
         return tmp;
     }
 
-    friend bool
+    friend constexpr bool
     operator==(const forward_iterator& x, const forward_iterator& y)
     {
         return x.it_ == y.it_;
     }
-    friend bool
+    friend constexpr bool
     operator!=(const forward_iterator& x, const forward_iterator& y)
     {
         return !(x == y);
@@ -205,14 +224,14 @@ class forward_iterator
 };
 
 template <class T, class U>
-inline bool
+inline bool constexpr
 operator==(const forward_iterator<T>& x, const forward_iterator<U>& y)
 {
     return x.base() == y.base();
 }
 
 template <class T, class U>
-inline bool
+inline bool constexpr
 operator!=(const forward_iterator<T>& x, const forward_iterator<U>& y)
 {
     return !(x == y);
@@ -227,35 +246,45 @@ class bidirectional_iterator
     friend class bidirectional_iterator;
 
   public:
-    typedef std::bidirectional_iterator_tag iterator_category;
-    typedef typename std::iterator_traits<It>::value_type value_type;
-    typedef typename std::iterator_traits<It>::difference_type difference_type;
-    typedef It pointer;
-    typedef typename std::iterator_traits<It>::reference reference;
+    typedef ::std::bidirectional_iterator_tag iterator_category;
+    typedef typename ::std::iterator_traits<It>::value_type value_type;
+    typedef typename ::std::iterator_traits<It>::difference_type difference_type;
+    typedef typename ::std::iterator_traits<It>::pointer pointer;
+    typedef typename ::std::iterator_traits<It>::reference reference;
 
-    It
+    constexpr It
     base() const
     {
         return it_;
     }
 
-    bidirectional_iterator() : it_() {}
-    explicit bidirectional_iterator(It it) : it_(it) {}
+    constexpr bidirectional_iterator() : it_() {}
+    explicit constexpr bidirectional_iterator(It it) : it_(it) {}
     template <class U>
-    bidirectional_iterator(const bidirectional_iterator<U>& u) : it_(u.it_)
+    constexpr bidirectional_iterator(const bidirectional_iterator<U>& u) : it_(u.it_)
     {
     }
 
-    reference operator*() const { return *it_; }
-    pointer operator->() const { return it_; }
+    constexpr reference operator*() const { return *it_; }
+    constexpr pointer operator->() const 
+    {
+        if constexpr (::std::is_pointer_v<It>)
+        {
+            return it_;
+        }
+        else
+        {
+            return it_.operator->();
+        }
+    }
 
-    bidirectional_iterator&
+    constexpr bidirectional_iterator&
     operator++()
     {
         ++it_;
         return *this;
     }
-    bidirectional_iterator
+    constexpr bidirectional_iterator
     operator++(int)
     {
         bidirectional_iterator tmp(*this);
@@ -263,13 +292,13 @@ class bidirectional_iterator
         return tmp;
     }
 
-    bidirectional_iterator&
+    constexpr bidirectional_iterator&
     operator--()
     {
         --it_;
         return *this;
     }
-    bidirectional_iterator
+    constexpr bidirectional_iterator
     operator--(int)
     {
         bidirectional_iterator tmp(*this);
@@ -277,19 +306,24 @@ class bidirectional_iterator
         return tmp;
     }
 
+    friend constexpr bool operator==(const bidirectional_iterator& x, const bidirectional_iterator& y)
+        {return x.it_ == y.it_;}
+    friend constexpr bool operator!=(const bidirectional_iterator& x, const bidirectional_iterator& y)
+        {return !(x == y);}
+
     template <class T>
     void operator,(T const &) DELETE_FUNCTION;
 };
 
 template <class T, class U>
-inline bool
+inline bool constexpr
 operator==(const bidirectional_iterator<T>& x, const bidirectional_iterator<U>& y)
 {
     return x.base() == y.base();
 }
 
 template <class T, class U>
-inline bool
+inline bool constexpr
 operator!=(const bidirectional_iterator<T>& x, const bidirectional_iterator<U>& y)
 {
     return !(x == y);
@@ -304,35 +338,45 @@ class random_access_iterator
     friend class random_access_iterator;
 
   public:
-    typedef std::random_access_iterator_tag iterator_category;
-    typedef typename std::iterator_traits<It>::value_type value_type;
-    typedef typename std::iterator_traits<It>::difference_type difference_type;
-    typedef It pointer;
-    typedef typename std::iterator_traits<It>::reference reference;
+    typedef ::std::random_access_iterator_tag iterator_category;
+    typedef typename ::std::iterator_traits<It>::value_type value_type;
+    typedef typename ::std::iterator_traits<It>::difference_type difference_type;
+    typedef typename ::std::iterator_traits<It>::pointer pointer;
+    typedef typename ::std::iterator_traits<It>::reference reference;
 
-    It
+    constexpr It
     base() const
     {
         return it_;
     }
 
-    random_access_iterator() : it_() {}
-    explicit random_access_iterator(It it) : it_(it) {}
+    constexpr random_access_iterator() : it_() {}
+    explicit constexpr random_access_iterator(It it) : it_(it) {}
     template <class U>
-    random_access_iterator(const random_access_iterator<U>& u) : it_(u.it_)
+    constexpr random_access_iterator(const random_access_iterator<U>& u) : it_(u.it_)
     {
     }
 
-    reference operator*() const { return *it_; }
-    pointer operator->() const { return it_; }
+    constexpr reference operator*() const { return *it_; }
+    constexpr pointer operator->() const 
+    {
+        if constexpr (::std::is_pointer_v<It>)
+        {
+            return it_;
+        }
+        else
+        {
+            return it_.operator->();
+        }
+    }
 
-    random_access_iterator&
+    constexpr random_access_iterator&
     operator++()
     {
         ++it_;
         return *this;
     }
-    random_access_iterator
+    constexpr random_access_iterator
     operator++(int)
     {
         random_access_iterator tmp(*this);
@@ -340,13 +384,13 @@ class random_access_iterator
         return tmp;
     }
 
-    random_access_iterator&
+    constexpr random_access_iterator&
     operator--()
     {
         --it_;
         return *this;
     }
-    random_access_iterator
+    constexpr random_access_iterator
     operator--(int)
     {
         random_access_iterator tmp(*this);
@@ -354,31 +398,31 @@ class random_access_iterator
         return tmp;
     }
 
-    random_access_iterator&
+    constexpr random_access_iterator&
     operator+=(difference_type n)
     {
         it_ += n;
         return *this;
     }
-    random_access_iterator
+    constexpr random_access_iterator
     operator+(difference_type n) const
     {
         random_access_iterator tmp(*this);
         tmp += n;
         return tmp;
     }
-    friend random_access_iterator
+    friend constexpr random_access_iterator
     operator+(difference_type n, random_access_iterator x)
     {
         x += n;
         return x;
     }
-    random_access_iterator&
+    constexpr random_access_iterator&
     operator-=(difference_type n)
     {
         return *this += -n;
     }
-    random_access_iterator
+    constexpr random_access_iterator
     operator-(difference_type n) const
     {
         random_access_iterator tmp(*this);
@@ -386,98 +430,98 @@ class random_access_iterator
         return tmp;
     }
 
-    reference operator[](difference_type n) const { return it_[n]; }
+    constexpr reference operator[](difference_type n) const { return it_[n]; }
 
     template <class T>
     void operator,(T const &) DELETE_FUNCTION;
 };
 
 template <class T, class U>
-inline bool
+inline bool constexpr
 operator==(const random_access_iterator<T>& x, const random_access_iterator<U>& y)
 {
     return x.base() == y.base();
 }
 
 template <class T, class U>
-inline bool
+inline bool constexpr
 operator!=(const random_access_iterator<T>& x, const random_access_iterator<U>& y)
 {
     return !(x == y);
 }
 
 template <class T, class U>
-inline bool
+inline bool constexpr
 operator<(const random_access_iterator<T>& x, const random_access_iterator<U>& y)
 {
     return x.base() < y.base();
 }
 
 template <class T, class U>
-inline bool
+inline bool constexpr
 operator<=(const random_access_iterator<T>& x, const random_access_iterator<U>& y)
 {
     return !(y < x);
 }
 
 template <class T, class U>
-inline bool
+inline bool constexpr
 operator>(const random_access_iterator<T>& x, const random_access_iterator<U>& y)
 {
     return y < x;
 }
 
 template <class T, class U>
-inline bool
+inline bool constexpr
 operator>=(const random_access_iterator<T>& x, const random_access_iterator<U>& y)
 {
     return !(x < y);
 }
 
 template <class T, class U>
-inline typename std::iterator_traits<T>::difference_type
+inline typename ::std::iterator_traits<T>::difference_type constexpr
 operator-(const random_access_iterator<T>& x, const random_access_iterator<U>& y)
 {
     return x.base() - y.base();
 }
 
 template <class Iter>
-inline Iter
+inline constexpr Iter
 base(output_iterator<Iter> i)
 {
     return i.base();
 }
 
 template <class Iter>
-inline Iter
+inline constexpr Iter
 base(input_iterator<Iter> i)
 {
     return i.base();
 }
 
 template <class Iter>
-inline Iter
+inline constexpr Iter
 base(forward_iterator<Iter> i)
 {
     return i.base();
 }
 
 template <class Iter>
-inline Iter
+inline constexpr Iter
 base(bidirectional_iterator<Iter> i)
 {
     return i.base();
 }
 
 template <class Iter>
-inline Iter
+inline constexpr Iter
 base(random_access_iterator<Iter> i)
 {
     return i.base();
 }
 
 template <class Iter> // everything else
-inline Iter
+inline constexpr Iter
 base(Iter i)
 {
     return i;
