@@ -14,10 +14,10 @@
 
 #include <oneapi/dpl/algorithm>
 
+#include "support/utils.h"
 #include "support/test_iterators.h"
 
 #include <cassert>
-#include <CL/sycl.hpp>
 
 template <typename _T1, typename _T2>
 void
@@ -67,7 +67,7 @@ test(sycl::queue& deviceQueue)
 int
 main()
 {
-    sycl::queue deviceQueue;
+    sycl::queue deviceQueue = TestUtils::get_test_queue();
     test<bidirectional_iterator<const int*>, bidirectional_iterator<int*>>(deviceQueue);
     test<bidirectional_iterator<const int*>, random_access_iterator<int*>>(deviceQueue);
     test<bidirectional_iterator<const int*>, int*>(deviceQueue);
@@ -77,5 +77,5 @@ main()
     test<const int*, bidirectional_iterator<int*>>(deviceQueue);
     test<const int*, random_access_iterator<int*>>(deviceQueue);
     test<const int*, int*>(deviceQueue);
-    return 0;
+    return TestUtils::done();
 }
