@@ -21,12 +21,12 @@ test_constexpr()
     {
     constexpr dpl::complex<T> lhs(1.5,  2.5);
     constexpr dpl::complex<T> rhs(1.5, -2.5);
-    STD_COMPLEX_TESTS_STATIC_ASSERT(lhs != rhs, "");
+    STD_COMPLEX_TESTS_STATIC_ASSERT(lhs != rhs);
     }
     {
     constexpr dpl::complex<T> lhs(1.5, 2.5);
     constexpr dpl::complex<T> rhs(1.5, 2.5);
-    STD_COMPLEX_TESTS_STATIC_ASSERT(!(lhs != rhs), "" );
+    STD_COMPLEX_TESTS_STATIC_ASSERT(!(lhs != rhs));
     }
 }
 
