@@ -13,21 +13,22 @@
 //   operator/(const T& lhs, const complex<T>& rhs);
 
 #include "support/test_complex.h"
+#include "./../cases.h"
 
 template <class T>
 void
 test(const T& lhs, const dpl::complex<T>& rhs, dpl::complex<T> x)
 {
-    assert(lhs / rhs == x);
+    assert(is_about(lhs / rhs, x));
 }
 
 template <class T>
 void
 test()
 {
-    T lhs(-8.5);
-    dpl::complex<T> rhs(1.5, 2.5);
-    dpl::complex<T>   x(-1.5, 2.5);
+    T lhs(-8.5f);
+    dpl::complex<T> rhs(1.5f, 2.5f);
+    dpl::complex<T>   x(-1.5f, 2.5f);
     test(lhs, rhs, x);
 }
 
