@@ -25,8 +25,8 @@ ONEDPL_TEST_NUM_MAIN
 
     IF_LONG_DOUBLE_SUPPORT(constexpr dpl::complex<long double> cd(2.5, 3.5);
                            constexpr dpl::complex<float> cf(cd);
-                           STD_COMPLEX_TESTS_STATIC_ASSERT(cf.real() == cd.real(), "");
-                           STD_COMPLEX_TESTS_STATIC_ASSERT(cf.imag() == cd.imag(), ""))
+                           STD_COMPLEX_TESTS_STATIC_ASSERT(cf.real() == cd.real());
+                           STD_COMPLEX_TESTS_STATIC_ASSERT(cf.imag() == cd.imag()))
 
   return 0;
 }
