@@ -16,12 +16,10 @@
 
 #include <oneapi/dpl/numeric>
 
-#include <CL/sycl.hpp>
 #include <iostream>
-#include "support/test_iterators.h"
 
-constexpr sycl::access::mode sycl_read = sycl::access::mode::read;
-constexpr sycl::access::mode sycl_write = sycl::access::mode::write;
+#include "support/utils.h"
+#include "support/test_iterators.h"
 
 template <typename _T1, typename _T2>
 void
@@ -38,7 +36,7 @@ template <class Iter1, class Iter2>
 void
 test()
 {
-    sycl::queue deviceQueue;
+    sycl::queue deviceQueue = TestUtils::get_test_queue();
     int input1[6] = {1, 2, 3, 4, 5, 6};
     int input2[6] = {6, 5, 4, 3, 2, 1};
     int output[8] = {};
@@ -50,9 +48,9 @@ test()
         sycl::buffer<int, 1> buffer2(input2, numOfItems1);
         sycl::buffer<int, 1> buffer3(output, numOfItems2);
         deviceQueue.submit([&](sycl::handler& cgh) {
-            auto in1 = buffer1.get_access<sycl_read>(cgh);
-            auto in2 = buffer2.get_access<sycl_read>(cgh);
-            auto out = buffer3.get_access<sycl_write>(cgh);
+            auto in1 = buffer1.get_access<sycl::access::mode::read>(cgh);
+            auto in2 = buffer2.get_access<sycl::access::mode::read>(cgh);
+            auto out = buffer3.get_access<sycl::access::mode::write>(cgh);
             cgh.single_task<KernelName<Iter1, Iter2>>([=]() {
                 out[0] = oneapi::dpl::inner_product(Iter1(&in1[0]), Iter1(&in1[0]), Iter2(&in2[0]), 0);
                 out[1] = oneapi::dpl::inner_product(Iter1(&in1[0]), Iter1(&in1[0]), Iter2(&in2[0]), 10);
@@ -108,6 +106,5 @@ main()
     test<const int*, const int*>();
 #endif
 
-    std::cout << "done" << std::endl;
-    return 0;
+    return TestUtils::done();
 }
